Option Explicit

'--------------------------- MODEMWARE ROUTINES ----------------------------

'-- ModemWare Copyright (c) 1992 - 1994 Crescent Software.
'   Written by Carl Franklin with help from and thanks to Bill Conley.

'-- Global flags. Set or read these in your application:
Global MW_CancelFlag%   '-- Setting to True cancels any ModemWare routine.
Global MW_DisplayFlag%  '-- Setting to True displays all received data.
Global MW_CurRoutine%   '-- Returns a value indicating the ModemWare
                        '   routine currently being processed. See the
                        '   MW_PROC_XXXX constants below.
Global MW_Capturing%     '-- Returns True if capturing to a file.
Global MW_ScriptRecording% '-- Returns True when recording a script file.

Global MW_SetupFlag%    '-- Internal Flag used by GetModemIndex

'-- Global reference to a comm control. used by ModemWare routines
'   that use VB forms (MWPORT.FRM, etc).
Global MW_Global_Comm1 As Control

Global MW_Init%

'-- The ModemWare internal error variable.
Dim MW_ErrCode%

'-- Left over data from WaitFor routine
Dim MW_LeftOver$

'-- API Calls & constants used by ModemWare.
'-- They are Aliased to avoid confusion with other modules that may use them.
Global Const SWP_NOMOVE = &H2
Global Const SWP_NOSIZE = &H1
Declare Sub MWSetWindowPos Lib "USER" Alias "SetWindowPos" (ByVal hWnd%, ByVal hWndInsertAfter%, ByVal X%, ByVal Y%, ByVal cx%, ByVal cy%, ByVal wFlags%)
Declare Function MWMoveTo& Lib "GDI" Alias "MoveTo" (ByVal hDC%, ByVal X%, ByVal Y%)
Declare Function MWLineTo Lib "GDI" Alias "LineTo" (ByVal hDC%, ByVal X%, ByVal Y%) As Integer
Declare Function MWGetWinDir Lib "Kernel" Alias "GetWindowsDirectory" (ByVal lpBuffer As String, ByVal nSize As Integer) As Integer
Declare Function MWGetConfigString Lib "Kernel" Alias "GetPrivateProfileString" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Integer, ByVal lpFileName As String) As Integer
Declare Function MWPutConfigString Lib "Kernel" Alias "WritePrivateProfileString" (ByVal lpApplicationName As String, ByVal lpKeyName As String, ByVal lpString As String, ByVal lplFileName As String) As Integer
Declare Function MWGetConfigInt Lib "Kernel" Alias "GetPrivateProfileInt" (ByVal lpApplicationName As String, ByVal lpKeyName As String, ByVal nDefault As Integer, ByVal lpFileName As String) As Integer

'-- These values are used all the time in comm programming. They are
'   automatically set when you call any ModemWare routine.
Global MW_CR$      '-- Carriage return
Global MW_LF$      '-- Line feed
Global MW_CRLF$    '-- Carriage return / Line feed
Global MW_ESC$     '-- Escape character
Global MW_BS$      '-- BackSpace Character
Global MW_SP$      '-- Space Character

'-- Result Codes (from the modem)
Global Const MW_RESULT_OK = -1000           '-- OK
Global Const MW_RESULT_CARRIER = -1001      '-- TIMEOUT
Global Const MW_RESULT_CONNECT = -1002      '-- CONNECT
Global Const MW_RESULT_RING = -1003         '-- RING
Global Const MW_RESULT_NO_CARRIER = -1004   '-- NO CARRIER
Global Const MW_RESULT_ERROR = -1005        '-- ERROR
Global Const MW_RESULT_NO_DIALTONE = -1006  '-- NO DIALTONE
Global Const MW_RESULT_BUSY = -1007         '-- BUSY
Global Const MW_RESULT_NO_ANSWER = -1008    '-- NO ANSWER
Global Const MW_RESULT_TIMEOUT = -1009      '-- TIMEOUT

'-- Status Messages
Global Const MW_STAT_SCRIPT_INITIALIZING = 5201     '-- Script: Initializing the Port or Settings.
Global Const MW_STAT_SCRIPT_WAITFOR = 5202          '-- Script: Waiting for a string.
Global Const MW_STAT_SCRIPT_SEND = 5203             '-- Script: Sending a string.
Global Const MW_STAT_SCRIPT_PORTSET = 5204          '-- Script: Settings have been specified.
Global Const MW_STAT_SCRIPT_CAPTURING = 5205        '-- Script: Capturing to a file.
Global Const MW_STAT_SCRIPT_DIALING = 5206          '-- Script: Dialing a number.
Global Const MW_STAT_SCRIPT_HANGUP = 5207           '-- Script: Hanging up.
Global Const MW_STAT_SCRIPT_UPLOAD = 5208           '-- Script: Uploading
Global Const MW_STAT_SCRIPT_DOWNLOAD = 5209         '-- Script: Downloading
Global Const MW_STAT_SCRIPT_REDIALING = 5210        '-- Script: Redialing a number
Global Const MW_STAT_SCRIPT_INPUT = 5211            '-- Script: Waiting for input
Global Const MW_STAT_PHBOOK_DIALING = 5301          '-- Phonebook: Dialing a number
Global Const MW_STAT_PHBOOK_REDIAL = 5302           '-- Phonebook: Redialing a number

'-- Value of MW_CurRoutine (Currently excecuting routine).
Global Const MW_PROC_NONE = 0               '-- No ModemWare routines are executing
Global Const MW_PROC_ASCIISEND = -50        '-- AsciiSend
Global Const MW_PROC_HANGUP = -51           '-- Hangup
Global Const MW_PROC_INITMODEM = -52        '-- InitModem
Global Const MW_PROC_MWEND = -56            '-- MWEnd
Global Const MW_PROC_MWPAUSE = -57          '-- MWPause
Global Const MW_PROC_OPENPORT = -58         '-- OpenPort
Global Const MW_PROC_PLACECALL = -59        '-- PlaceCall
Global Const MW_PROC_SENDMODEMCMD = -60     '-- SendModemCmd
Global Const MW_PROC_WAITFOR = -61          '-- WaitFor         Note: This is only set if YOU
                                            '                         call WaitFor directly.
Global Const MW_PROC_WAITFORA = -62         '-- WaitForA        Note: This is only set if YOU
                                            '                         call WaitForA directly.
Global Const MW_PROC_WAITFORCALL = -63      '-- WaitForCall
Global Const MW_PROC_SETMODEMSETTINGS = -64
Global Const MW_PROC_LOADPORTSETTINGS = -65
Global Const MW_PROC_SETPORTSETTINGS = -66
Global Const MW_PROC_END = -70

'-- General errors returned by some modemware routines
Global Const MW_ER_CANCELED = -102          '-- The process was Canceled by the user setting
                                            '   MW_CancelFlag to True.
Global Const MW_ER_LOST_CARRIER = -103      '-- Carrier was lost. The call was disconnected.
Global Const MW_ER_PORTCLOSED = -104        '-- The specified port does not exist
Global Const MW_ER_CTRLERROR = -105         '-- The specified port can not be accessed
Global Const MW_ER_TIMEOUT = -106           '-- Timeout
Global Const MW_ER_FILE_ERR = -107

'-- AsciiSend
Global Const MW_ASCIISEND_OK = 100          '-- File sent ok.
Global Const MW_ASCIISEND_NOTFOUND = 101    '-- File not found.
Global Const MW_ASCIISEND_FILE_ERR = 102    '-- Error reading file.

'-- GetCommMessage errors
'Global Const MW_GETMSG_OK = 200             '-- Valid CRESCENT MESSAGE
'Global Const MW_GETMSG_INVALID_MSG = 201    '-- Invalid CRESCENT MESSAGE

'-- GetModemIndex errors
Global Const MW_GETMODEMINDEX_ERROR = 250   '-- No Current Modems, call GetModemSettings

'-- Hangup Errors
Global Const MW_HANGUP_OK = 300             '-- Modem disconnected
Global Const MW_HANGUP_ERROR = 301          '-- Modem could not be disconnected
Global Const MW_HANGUP_OFFLINE = 302        '-- Modem is already hung up

'-- InitModem Errors
Global Const MW_INITMODEM_OK = 400          '-- Modem initialized OK.
Global Const MW_INITMODEM_NO_MODEM = 401    '-- No modem found.
Global Const MW_INITMODEM_ONLINE = 402      '-- Modem is online and cannot be initialized.
Global Const MW_INITMODEM_BAD_PORT = 403    '-- Invalid Port number specified
Global Const MW_INITMODEM_BAD_SETTINGS = 404'-- Invalid Settings specified
Global Const MW_INITMODEM_BAUD_DRIVER = 405 '-- Specified baud rate is not supported by the comm control/driver.
Global Const MW_INITMODEM_BAUD_MODEM = 406  '-- Specified baud rate is not supported by your modem.

'-- OpenPort Errors
Global Const MW_OPENPORT_OK = 500           '-- Port opened ok
Global Const MW_OPENPORT_ERROR = 501        '-- No ports are avaiable
Global Const MW_OPENPORT_SETTINGS = 502     '-- The specified settings are invalid.

'-- PlaceCall Errors
Global Const MW_PLACECALL_OK = 600          '-- A valid result code was returned. Check Result
Global Const MW_PLACECALL_ERROR = 601       '-- The modem reported an error
Global Const MW_PLACECALL_NO_ANSWER = 602   '-- There is no answer
Global Const MW_PLACECALL_NO_PORT = 603   '-- There is no answer

'-- SendCommMsg Errors
Global Const MW_SENDMSG_OK = 700            '-- Valid CRESCENT MESSAGE sent.

'-- SendModemCmd Errors
Global Const MW_SENDMODEMCMD_OK = 800       '-- Command sent ok.
Global Const MW_SENDMODEMCMD_NOCMD = 801    '-- No command was specified.

'-- WaitFor Errors
Global Const MW_WAITFOR_OK = 900            '-- The wait string was received

'-- WaitForCall Errors
Global Const MW_WAITFORCALL_OK = 1000       '-- WaitForCall returned OK, caller is connected.

'-------------------------- Modem Database Stuff ---------------------------

'-- MW_ModemInfo is an array of this type which contains modem data
'   for more more than 400 modems. Read from MODEMS.DAT
'   There is a 4 byte header on MODEMS.DAT which contains the number of
'   records in the database.
Type MW_ModemInfoType
    DeletedFlag As Integer
    LinkRecord As Long
    ModemNumber As Long
    VendorName As String * 25
    ModemName As String * 40
    InitString As String * 128
    Attention As String * 30
    Hangup As String * 50
    Reset As String * 30
    Answer As String * 30
    Dial As String * 30
    Busy As String * 30
    HighestBaud As Long
    Connect As String * 40
End Type

'-- Length of the MW_ModemInfoType structure.
Global Const LenModemType = 447

'-- MW_CurModem contains the current modem's information.
Global MW_CurModem() As MW_ModemInfoType

'-- MW_CurModemIndex contains the Tag properties of each of the
'   comm controls in your project.
Global MW_CurModemIndex() As String

'---------------------------- Script Stuff -----------------------------

'-- Internal: Length of each recorded "WAITFOR" String
'   when recording a script.
Global MW_ScriptWaitLen

'-- Internal: Used to store script variables.
Type MW_ScriptVariableType
    Variable As String
    Value As String
End Type
Global MW_ScriptVars() As MW_ScriptVariableType

'-- Used to store script labels.
Type MW_ScriptLabelType
    Label As String
    LineNum As Integer
End Type
Global MW_ScriptLabel() As MW_ScriptLabelType

'-- Script data is stored in an array of this type.
Type MW_ScriptDataType
    Cmd As String
    Dat As String
End Type
Global MW_RecScriptData() As MW_ScriptDataType

'-- Script Commands
Global Const MW_SCRIPT_CMD_PORT = 5000          '-- Set the port number.
Global Const MW_SCRIPT_CMD_SETTINGS = 5001      '-- Set the settings.
Global Const MW_SCRIPT_CMD_DIAL = 5002          '-- Dial a number.
Global Const MW_SCRIPT_CMD_TIMEOUT = 5003       '-- Set the Timeout value.
Global Const MW_SCRIPT_CMD_WAITFOR = 5004       '-- Wait for a string.
Global Const MW_SCRIPT_CMD_SEND = 5005          '-- Send a string.
Global Const MW_SCRIPT_CMD_HANGUP = 5006        '-- Hang up the phone.
Global Const MW_SCRIPT_CMD_PROTOCOL = 5007      '-- Set the protocol.
Global Const MW_SCRIPT_CMD_UPLOAD = 5008        '-- Upload (send) a file.
Global Const MW_SCRIPT_CMD_DOWNLOAD = 5009      '-- Download (receive) a file.
Global Const MW_SCRIPT_CMD_PAUSE = 5010         '-- Pause for a number of timer ticks.
Global Const MW_SCRIPT_CMD_STOP = 5011          '-- Stops execution of your app (for debugging).
Global Const MW_SCRIPT_CMD_END = 5012           '-- Ends the script.
Global Const MW_SCRIPT_CMD_INPUT = 5013         '-- Asks the user to enter a variable.
Global Const MW_SCRIPT_CMD_CAPTURE = 5014       '-- Captures data to a disk file.
Global Const MW_SCRIPT_CMD_CLOSECAPTURE = 5015  '-- Closes the capture file
Global Const MW_SCRIPT_CMD_LABEL = 5016         '-- Defines a label. I.E. ":JumpHere"
Global Const MW_SCRIPT_CMD_ONTIMEOUTGOTO = 5017 '-- Determines where to branch if a timeout occurs.

'-- ScriptPlay Errors
Global Const MW_SCRIPTPLAY_OK = 5100            '-- Script file ended normally.
Global Const MW_SCRIPTPLAY_BAD_FILE = 5101      '-- An invalid script filename was specified.
Global Const MW_SCRIPTPLAY_BAD_PORT = 5102      '-- An Invalid port was specified.
Global Const MW_SCRIPTPLAY_BAD_SETTINGS = 5103  '-- Invalid settings were specified.
Global Const MW_SCRIPTPLAY_BAD_PROTOCOL = 5104  '-- An invalid protocol was specified.
Global Const MW_SCRIPTPLAY_BAD_XFERFILE = 5105  '-- An invalid filename was specified to Upload or Download.
Global Const MW_SCRIPTPLAY_BAD_COMMAND = 5106   '-- An invalid script command was specified
Global Const MW_SCRIPTPLAY_NOTSUP = 5107        '-- The specified command is not supported by the comm control
Global Const MW_SCRIPTPLAY_NO_PORT = 5108       '-- No port was specified.
Global Const MW_SCRIPTPLAY_NOPROTOCOL = 5109    '-- Tried to transfer a file with no protocol set.

Function MWError ()
'-- Returns the current ModemWare error.

    MWError = MW_ErrCode

End Function

Function MWErrorMsg$ ()
'-- Returns an error string based on the current ModemWare Error

    Select Case MWError()
        '-- General Errors
        Case MW_ER_CANCELED
            MWErrorMsg$ = "Process Canceled By User"
        Case MW_ER_LOST_CARRIER
            MWErrorMsg$ = "Carrier Lost. The Call Has Been Disconnected"
        Case MW_ER_PORTCLOSED
            MWErrorMsg$ = "The Specified Port Is Not Open"
        Case MW_ER_CTRLERROR
            MWErrorMsg$ = "The Specified Comm Control Can Not Be Accessed"
        Case MW_ER_TIMEOUT
            MWErrorMsg$ = "Timeout Error"
        
        '-- AsciiSend Errors
        Case MW_ASCIISEND_OK
            MWErrorMsg$ = "File sent ok."
        Case MW_ASCIISEND_NOTFOUND
            MWErrorMsg$ = "File not found."
        Case MW_ASCIISEND_FILE_ERR
            MWErrorMsg$ = "Error reading file."
        
        '-- GetModemIndex Errors
        Case MW_GETMODEMINDEX_ERROR
            MWErrorMsg$ = "No Current Modems. Please configure your modem."
        
        '-- GetCommMessage Errors
        'Case MW_GETMSG_OK
        '    MWErrorMsg$ = "Valid Message Received"
        'Case MW_GETMSG_INVALID_MSG
        '    MWErrorMsg$ = "Invalid Message Received"
        
        '-- Hangup Errors
        Case MW_HANGUP_OK
            MWErrorMsg$ = "Modem Disconnected"
        Case MW_HANGUP_ERROR
            MWErrorMsg$ = "Modem Could Not Be Disconnected"
        Case MW_HANGUP_OFFLINE
            MWErrorMsg$ = "Modem Is Not Online"
        
        '-- InitModem Errors
        Case MW_INITMODEM_OK
            MWErrorMsg$ = "Modem Initialized"
        Case MW_INITMODEM_NO_MODEM
            MWErrorMsg$ = "Invalid Baud Rate, No Modem Found, Or Modem Is Turned Off Or Not Connected"
        Case MW_INITMODEM_ONLINE
            MWErrorMsg$ = "Modem Is Online And Cannot Be Initialized"
        Case MW_INITMODEM_BAD_PORT
            MWErrorMsg$ = "Invalid Port number Specified"
        Case MW_INITMODEM_BAD_SETTINGS
            MWErrorMsg$ = "Invalid Settings Specified"
        Case MW_INITMODEM_BAUD_DRIVER
            MWErrorMsg$ = "Specified baud rate is not supported by the comm control/driver"
        Case MW_INITMODEM_BAUD_MODEM
            MWErrorMsg$ = "Specified baud rate is not supported by your modem"

        '-- OpenPort Errors
        Case MW_OPENPORT_OK
            MWErrorMsg$ = "Port opened ok"
        Case MW_OPENPORT_ERROR
            MWErrorMsg$ = "No ports are avaiable"
        Case MW_OPENPORT_SETTINGS
            MWErrorMsg$ = "The specified settings are invalid"
        
        '-- PlaceCall Errors
        Case MW_PLACECALL_OK
            MWErrorMsg$ = "A valid result code was returned. Check Result"
        
        '-- ScriptPlay Errors
        Case MW_SCRIPTPLAY_OK
            MWErrorMsg$ = "Script Completed"
        Case MW_SCRIPTPLAY_BAD_FILE
            MWErrorMsg$ = "An invalid script filename was specified."
        Case MW_SCRIPTPLAY_BAD_PORT
            MWErrorMsg$ = "An Invalid port was specified."
        Case MW_SCRIPTPLAY_BAD_SETTINGS
            MWErrorMsg$ = "Invalid settings were specified."
        Case MW_SCRIPTPLAY_BAD_PROTOCOL
            MWErrorMsg$ = "An invalid protocol was specified."
        Case MW_SCRIPTPLAY_BAD_XFERFILE
            MWErrorMsg$ = "An invalid transfer filename was specified."
        Case MW_SCRIPTPLAY_BAD_COMMAND
            MWErrorMsg$ = "Invalid script command, or syntax error."
        Case MW_SCRIPTPLAY_NOTSUP
            MWErrorMsg$ = "The specified command is not supported by the comm control."
        Case MW_PLACECALL_NO_PORT
            MWErrorMsg$ = "No port was specified."
        Case MW_SCRIPTPLAY_NOPROTOCOL
            MWErrorMsg$ = "Tried to transfer a file with no protocol set."
        
        '-- SendCommMsg Errors
        Case MW_SENDMSG_OK
            MWErrorMsg$ = "Valid CRESCENT MESSAGE sent."
        Case MW_SENDMODEMCMD_NOCMD
            MWErrorMsg$ = "No command was specified."
        
        '-- SendModemCmd Errors
        Case MW_SENDMODEMCMD_OK
            MWErrorMsg$ = "Command sent ok."
        
        '-- WaitFor Errors
        Case MW_WAITFOR_OK
            MWErrorMsg$ = "The wait string was received"

        '-- WaitForCall Errors
        Case MW_WAITFORCALL_OK
            MWErrorMsg$ = "Caller Is Connected"
    End Select

End Function

Sub MWPause (Secs!)
'-- Pauses for a number of seconds (or fractions of a second)

Dim PrevRoutine As Integer
Dim Start!, EndTime!
    
    '-- Tell everybody where we are
    PrevRoutine = MW_CurRoutine
    MW_CurRoutine = MW_PROC_HANGUP
    
    Start! = Timer
    EndTime! = Start! + Secs!
    
    Do
        DoEvents
        If Start! > Timer Then
            '-- Adjust for Midnight
            EndTime! = EndTime! - 24 * 60 * 60
        End If
    Loop Until Timer >= EndTime!
    
    MW_CurRoutine = PrevRoutine

End Sub

Sub MWSetError (Code)
'-- Sets the current ModemWare error

    MW_ErrCode = Code

End Sub

Sub MWStatus (StatusCode, ExtraData$)
'-- This subroutine is called by ModemWare periodically to let the
'   programmer know what's happenning.

'   StatusCode contains a value, which is used to derive a status message string.

'   ExtraData$ holds different information depending on the StatusCode.
'   See the PDQComm for Windows documentation for a table of codes and meanings

'   This routine serves as a handy way to update a status bar, but it also
'   allow you to trap the code for your own purposes, be they debugging or
'   development.

Dim Msg$
    
    Select Case StatusCode
        Case MW_STAT_SCRIPT_INITIALIZING
            Msg$ = "Script: Initializing the port.."
        Case MW_STAT_SCRIPT_WAITFOR
            Msg$ = "Script: Waiting For " & ExtraData$
        Case MW_STAT_SCRIPT_SEND
            Msg$ = "Script: Sending " & ExtraData$
        Case MW_STAT_SCRIPT_PORTSET
            Msg$ = "Script: Port is set at " & ExtraData$
        Case MW_STAT_SCRIPT_CAPTURING
            Msg$ = "Script: Capturing to " & ExtraData$
        Case MW_STAT_SCRIPT_DIALING
            Msg$ = "Script: Dialing " & ExtraData$
        Case MW_STAT_SCRIPT_HANGUP
            Msg$ = "Script: Hanging up"
        Case MW_STAT_SCRIPT_UPLOAD
            Msg$ = "Script: Uploading " & ExtraData$
        Case MW_STAT_SCRIPT_DOWNLOAD
            Msg$ = "Script: Downloading " & ExtraData$
        Case MW_STAT_SCRIPT_REDIALING
            Msg$ = "Script: Redial #" & ExtraData$
        Case MW_STAT_PHBOOK_DIALING
            Msg$ = "Dialing " & ExtraData$
        Case MW_STAT_PHBOOK_REDIAL
            Msg$ = "Redial Attempt # " & ExtraData$
        Case 0
            '-- If StatusCode is zero, just print the ExtraData$
            Msg$ = ExtraData$
    End Select
    
    '-- StatusPrint is a routine in CTRLDEMO.BAS that prints a message on the CTRLDEMO
    '   status bar. You can change this line to print the message anywhere, or just
    '   comment it out. We advise that you write your own StatusPrint subroutine.
    
    StatusPrint Msg$

End Sub

Function OpenPort (Comm1 As Control, PortNum, BaudRate&, Settings$)

Dim First As Integer, I As Integer

'-- This routine opens the first available comm port.

    '-- Settings
    On Error Resume Next
    Comm1.Settings = Settings$
    If Err Then
        MWSetError MW_OPENPORT_SETTINGS
        Exit Function
    End If
    On Error GoTo 0

    '-- Tell everybody where we are
    If Not MW_CurRoutine Then
        MW_CurRoutine = MW_PROC_OPENPORT
    End If
    
    '-- Was PortNum not specified?
    If PortNum = 0 Then
        First = 1
    Else
        First = PortNum
    End If
    
    For I = First To 9
        
        If MW_CancelFlag Then
            MWSetError MW_ER_CANCELED
            GoTo OPExit
        End If

        '-- Check ports 1 through 9
        Comm1.CommPort = I
        '-- Trap VB errors
        On Error Resume Next
        '-- Try to open the port
        Comm1.PortOpen = True
        If Err = 0 Then
            OpenPort = I
            Comm1.PortOpen = False
            MWSetError MW_OPENPORT_OK
            Exit For
        Else
            MWSetError MW_OPENPORT_ERROR
        End If
        OpenPort = 0
    Next

OPExit:

    MW_CurRoutine = MW_PROC_NONE

End Function

Function RemoveCaret$ (CaretString$)
'-- Translates caret codes into ascii characters
    
Dim Caret As Integer, L As Integer
Dim Char$, Temp$

    Do
        Caret = InStr(CaretString$, "^")
        L = Len(CaretString$)
        If Caret Then
            If Caret < L Then
                Char$ = Chr$(Asc(Mid$(CaretString$, Caret + 1, 1)) - 64)
                Temp$ = Left$(CaretString$, Caret - 1) & Char$
                If Caret + 2 <= L Then
                    Temp$ = Temp$ & Mid$(CaretString$, Caret + 2)
                End If
                CaretString$ = Temp$
            End If
        Else
            Exit Do
        End If
    Loop
    RemoveCaret$ = CaretString$

End Function

Function WaitFor$ (Comm1 As Control, Wait$, TimeOut%, CDFlag%, ExcludeFlag%)
'-- Waits X number of seconds to receive a string or character.
'   This can be used with any hardware, not just modems, as a means of
'   receiving a particular string. For example, if you are connected to
'   a machine that sends a string ending in a Chr$(13), you can call WaitFor
'   with Wait$ = Chr$(13). It will return all data received up to the Chr$(13).

Dim Test As Integer, AutoP As Integer, RT As Integer, OldLen As Integer, There As Integer
Dim Start!, EndTime!, NewData$, In$, DispStr$

    '-- Test Comm port validity
    On Error Resume Next
    Test = Comm1.CommPort
    If Err Then
        '-- Port can not be accessed
        MWSetError MW_ER_CTRLERROR
        Exit Function
    Else
        If Comm1.PortOpen = False Then
            '-- Port it closed
            MWSetError MW_ER_PORTCLOSED
            Exit Function
        End If
    End If
    On Error GoTo 0
    
    '-- InitModemWare initializes MW_CR$, MW_LF$, etc.
    Call InitModemWare
    
    '-- Initialize MW_CancelFlag
    MW_CancelFlag = False
    
    '-- Tell everybody where we are
    If Not MW_CurRoutine Then
        MW_CurRoutine = MW_PROC_WAITFOR
    End If
    
    '-- Turn Off AutoProcess & RThreshold
    AutoP = Comm1.AutoProcess
    If AutoP Then
        Comm1.AutoProcess = PDQ_AUTOPROCESS_NONE
    End If
    RT = Comm1.RThreshold
    If RT Then
        Comm1.RThreshold = 0
    End If

    '-- Display without emulation?
    If MW_DisplayFlag Then
        If Comm1.Emulation = 0 Then
            MW_DisplayFlag = False
        End If
    End If

    '-- InputLen must be 1
    OldLen = Comm1.InputLen
    If OldLen <> 0 Then
        Comm1.InputLen = 0
    End If

    '-- Initialize variables just in case they were passed in
    MWSetError False
    In$ = MW_LeftOver$
    
    Start! = Timer
    '-- Check for time to wrap around midnight
    If Timer + TimeOut > 86400 Then
        EndTime! = TimeOut - (86400 - Timer)
    Else
        EndTime! = Start! + TimeOut
    End If
    
    '-- And they're off!
    Do
        '-- Have we received any data?
        If Comm1.InBufferCount Then
            '-- Get the data
            NewData$ = Comm1.Input
            DoEvents
            '-- Add it to In$
            In$ = In$ & NewData$
            
            '-- Add it to a display string (if MW_DisplayFlag)
            If MW_DisplayFlag Then
                DispStr$ = DispStr$ & NewData$
                If Len(DispStr$) > 32 Then
                    Comm1.Disp = DispStr$
                    DoEvents
                    DispStr$ = ""
                End If
            End If
            
            '-- Did we get what we're waiting for?
            There = InStr(In$, Wait$)
            If There Then
                If MW_DisplayFlag Then
                    Comm1.Disp = DispStr$
                    DoEvents
                End If
                If ExcludeFlag Then
                    WaitFor$ = Left$(In$, There - 1)
                Else
                    WaitFor$ = Left$(In$, There + Len(Wait$))
                End If
                MW_LeftOver$ = Mid$(In$, There + Len(Wait$))
                MWSetError MW_WAITFOR_OK
                Exit Do
            End If
            
            '-- Is In$ getting too big?
            If Len(In$) > 32000 Then
                '-- Trim it
                In$ = Right$(In$, Len(Wait$))
            End If
        Else
            '-- No new data, display DispStr$
            If Len(DispStr$) Then
                Comm1.Disp = DispStr$
                DoEvents
                DispStr$ = ""
            End If
        End If

        '-- Have we been Canceled?
        If MW_CancelFlag Then
            MW_CancelFlag = False
            MWSetError MW_ER_CANCELED
            WaitFor$ = In$
            Exit Do
        End If
        
        '-- Check Carrier
        If CDFlag Then
            If Comm1.CDHolding = False Then
                MWSetError MW_ER_LOST_CARRIER
                WaitFor$ = In$
                Exit Do
            End If
        End If

        '-- Are we out of time
        If Timer >= EndTime! Then
            MWSetError MW_ER_TIMEOUT
            WaitFor$ = In$
            Exit Do
        End If
        
        DoEvents
        
    Loop

    '-- Reset InputLen
    If OldLen Then
        Comm1.InputLen = OldLen
    End If
    
WFExit:

    '-- Reset AutoProcess and RThreshold
    If AutoP Then
        Comm1.AutoProcess = AutoP
    End If
    If RT Then
        Comm1.RThreshold = RT
    End If
    
    MW_CurRoutine = MW_PROC_NONE

End Function

Function WaitForA$ (Comm1 As Control, Wait$(), TimeOut, CDFlag, ExcludeFlag, Found)
'-- Same as WaitFor$ (See WaitFor$) but accepts an array as an argument.
'   WaitForA$ returns when it finds any one of the specified strings.

Dim Test As Integer, AutoP As Integer, RT As Integer, NumEls As Integer
Dim FirstEl As Integer, OldLen As Integer, I As Integer, There As Integer
Dim NewData$, T$, In$, DispStr$, Start!, EndTime!

    '-- Test Comm port validity
    On Error Resume Next
    Test = Comm1.CommPort
    If Err Then
        '-- Port can not be accessed
        MWSetError MW_ER_CTRLERROR
        Exit Function
    Else
        If Comm1.PortOpen = False Then
            '-- Port it closed
            MWSetError MW_ER_PORTCLOSED
            Exit Function
        End If
    End If
    On Error GoTo 0
    
    '-- InitModemWare initializes MW_CR$, MW_LF$, etc.
    Call InitModemWare
    
    '-- Initialize MW_CancelFlag
    MW_CancelFlag = False
    
    '-- Tell everybody where we are
    If Not MW_CurRoutine Then
        MW_CurRoutine = MW_PROC_WAITFORA
    End If
    
    '-- Turn Off AutoProcess & RThreshold
    AutoP = Comm1.AutoProcess
    If AutoP Then
        Comm1.AutoProcess = PDQ_AUTOPROCESS_NONE
    End If
    RT = Comm1.RThreshold
    If RT Then
        Comm1.RThreshold = 0
    End If
    
    '-- Number of strings
    On Error Resume Next
    NumEls = UBound(Wait$)
    If Err Then GoTo WFAExit
    FirstEl = LBound(Wait$)

    '-- Display without emulation?
    If MW_DisplayFlag Then
        If Comm1.Emulation = 0 Then
            MW_DisplayFlag = False
        End If
    End If

    '-- InputLen must be 1
    OldLen = Comm1.InputLen
    If OldLen <> 0 Then
        Comm1.InputLen = 0
    End If

    '-- Initialize variables just in case they were passed in
    MWSetError False
    Found = -1
    
    Start! = Timer
    '-- Check for time to wrap around midnight
    If Timer + TimeOut > 86400 Then
        EndTime! = TimeOut - (86400 - Timer)
    Else
        EndTime! = Start! + TimeOut
    End If
    
    '-- And they're off!
    In$ = MW_LeftOver$
    Do
        '-- Have we received any data?
        If Comm1.InBufferCount Then
            '-- Get the data
            DoEvents
            NewData$ = Comm1.Input
            '-- Add it to In$
            In$ = In$ & NewData$
            
            '-- Add it to a display string (if MW_DisplayFlag)
            If MW_DisplayFlag Then
                DispStr$ = DispStr$ & NewData$
                If Len(DispStr$) > 32 Then
                    Comm1.Disp = DispStr$
                    DoEvents
                    DispStr$ = ""
                End If
            End If
            
            '-- Did we get what we're waiting for?
            For I = FirstEl To NumEls
                There = InStr(In$, Wait$(I))
                If There Then Exit For
            Next
            If There Then
                If MW_DisplayFlag Then
                    Comm1.Disp = DispStr$
                    DoEvents
                End If
                If ExcludeFlag Then
                    WaitForA$ = Left$(In$, There - 1)
                Else
                    WaitForA$ = Left$(In$, There + Len(Wait$(I)) - 1)
                End If
                MW_LeftOver$ = Mid$(In$, There + Len(Wait$(I)))
                Found = I
                MWSetError MW_WAITFOR_OK
                Exit Do
            End If
            
            '-- Is In$ getting too big?
            If Len(In$) > 32000 Then
                '-- Trim it
                In$ = Right$(In$, 3200)
            End If
        
        Else
            '-- No new data, display DispStr$
            If Len(DispStr$) Then
                Comm1.Disp = DispStr$
                DoEvents
                DispStr$ = ""
            End If
            
        End If

        '-- Have we been Canceled?
        If MW_CancelFlag Then
            MW_CancelFlag = False
            MWSetError MW_ER_CANCELED
            Exit Do
        End If
        
        '-- Check Carrier
        If CDFlag Then
            If Comm1.CDHolding = False Then
                MWSetError MW_ER_LOST_CARRIER
                Exit Do
            End If
        End If

        '-- Are we out of time
        If Timer >= EndTime! Then
            MWSetError MW_ER_TIMEOUT
            Exit Do
        End If
        
        '-- Most important
        DoEvents
        
    Loop

    '-- Reset InputLen
    If OldLen Then
        Comm1.InputLen = OldLen
    End If
    
WFAExit:

    '-- Reset AutoProcess and RThreshold
    If AutoP Then
        Comm1.AutoProcess = AutoP
    End If
    If RT Then
        Comm1.RThreshold = RT
    End If
    
    MW_CurRoutine = MW_PROC_NONE

End Function

Function AddCaret$ (ConvertString$)

Dim I As Integer, CharVal As Integer
Dim Char$, Temp$

'-- This function translates caret codes into ascii characters
    
    For I = 1 To Len(ConvertString$)
        Char$ = Mid$(ConvertString$, I, 1)
        CharVal = Asc(UCase$(Char$))
        If CharVal < 32 And CharVal > 0 Then
            Temp$ = Temp$ & "^" & Chr$(CharVal + 64)
        Else
            Temp$ = Temp$ & Char$
        End If
    Next
    
    AddCaret$ = Temp$

End Function

Sub InitModemWare ()
'-- Called by all ModemWare routines

    MW_CR$ = Chr$(13)
    MW_LF$ = Chr$(10)
    MW_CRLF$ = Chr$(13) & Chr$(10)
    MW_ESC$ = Chr$(27)
    MW_BS$ = Chr$(8)
    MW_SP$ = Chr$(32)
    MW_Init = True

End Sub

Sub MWEnd (Comm1 As Control)
'-- Safely closes the port.

'   You should call this routine in your main Form_Unload event.
'   To end your program, unload your main form. ex: Unload Form1

    '-- Tell everyone where we are.
    If Not MW_CurRoutine Then
        MW_CurRoutine = MW_PROC_END
    End If
    
    '-- Cancel the current operation
    MW_CancelFlag = True
    MWPause .5
    
    '-- This routine makes sure your modem is disconnected and your comm port is closed.
    If Comm1.PortOpen Then
        Do
            DoEvents
        Loop Until Comm1.OutBufferCount = 0
        'Hangup Comm1, 1, False  '-- Don't Use DTR
        'Hangup Comm1, 1, True   '-- Use DTR
    End If
    
    MW_CurRoutine = MW_PROC_NONE

    '-- Are we recording a script?
    If MW_ScriptRecording Then
        AddToRecScriptData "END", ""
        Call ScriptRecord(Comm1, "", 10)
    End If

End Sub

