Option Explicit

'-- ModemWare Copyright (c) 1992 - 1994 Crescent Software.
'   Written by Carl Franklin with help from and thanks to Bill Conley.

Type MW_LinkRecordsType
    VendorName As String * 25
    Record As Long
End Type

Global MW_ModemInfo() As MW_ModemInfoType
Global CurRec&, DataFile$, NumBrands As Integer
Global MW_LinkRecords() As MW_LinkRecordsType
Global MW_NumRecords&

Function GetModemIndex (Comm1 As Control)
    
Dim NumModems As Integer, I As Integer
Dim T$

    On Error Resume Next
    NumModems = UBound(MW_CurModemIndex)
    If NumModems Then
        T$ = Comm1.Tag
        For I = 1 To NumModems
            If T$ = MW_CurModemIndex(I) Then
                GetModemIndex = I
                Exit Function
            End If
        Next
    End If
    
    '-- This control is not registered in the MWCurModemIndex array.
    '   So add it.
    NumModems = NumModems + 1
    ReDim Preserve MW_CurModemIndex(1 To NumModems) As String
    ReDim Preserve MW_CurModem(1 To NumModems) As MW_ModemInfoType
    MW_CurModemIndex(NumModems) = Comm1.Tag
    GetModemIndex = NumModems

End Function

Sub Hangup (Comm1 As Control, Attempts%, UseDTR%)
'-- Hangs up (disconnects) the modem.
Dim Test%, AutoP%, RT%, I%, CDFlag%, ExcludeFlag%, NoGood%
Dim ModemNum%, Result%
Dim Received$, HangUpCmd$

    On Error Resume Next
    Test = Comm1.CommPort
    If Err Then
        '-- Port can not be accessed
        MWSetError MW_ER_CTRLERROR
        Exit Sub
    Else
        If Comm1.PortOpen = False Then
            '-- Port it closed
            MWSetError MW_ER_PORTCLOSED
            Exit Sub
        End If
        If Comm1.CDHolding = False Then
            MWSetError MW_HANGUP_OFFLINE
            Exit Sub
        End If
    End If
    On Error GoTo 0

    '-- InitModemWare initializes MW_CR$, MW_LF$, etc.
    Call InitModemWare

    '-- Initialize return variables
    MWSetError 0

    '-- Tell everybody where we are
    If Not MW_CurRoutine Then
        MW_CurRoutine = MW_PROC_Hangup
    End If

    '-- Turn Off AutoProcess & RThreshold
    AutoP = Comm1.AutoProcess
    If AutoP Then
        Comm1.AutoProcess = PDQ_AUTOPROCESS_NONE
    End If
    RT = Comm1.RThreshold
    If RT Then
        Comm1.RThreshold = 0
    End If

    '-- Are we recording a script?
    If MW_ScriptRecording Then
        AddToRecScriptData "HANGUP", ""
    End If
    
    '-- Status print
    MWStatus 0, "Hanging Up..."
    
    '-- Use DTR?
    If UseDTR Then
        Comm1.DTREnable = False
        MWPause .5
        Comm1.DTREnable = True
        If Comm1.CDHolding Then
            MWSetError MW_HANGUP_ERROR
        Else
            MWSetError MW_HANGUP_OK
        End If
        GoTo HUExit
    End If

    '-- Was 0 passed in?
    If Attempts = 0 Then Attempts = 1

    For I = 1 To Attempts
        '-- Send the escape sequence
        MWPause 2
        Comm1.Output = "+++"
        MWPause 2

        '-- Wait For 'OK' from the modem
        CDFlag = True
        ExcludeFlag = True
        Received$ = WaitFor$(Comm1, "OK", 7, CDFlag, ExcludeFlag)
        
        Select Case MWError()
            Case MW_ER_TIMEOUT, MW_ER_CANCELED
                GoTo HUExit
            Case MW_ER_LOST_CARRIER
                MWSetError MW_HANGUP_OK
                GoTo HUExit
            Case MW_WAITFOR_OK
                Exit For
            Case Else
                If I = Attempts Then
                    NoGood = True
                End If
        End Select
    Next

    If NoGood Then
        MWSetError MW_HANGUP_ERROR
        GoTo HUExit
    End If

    '-- Get the current modem index
    ModemNum = GetModemIndex(Comm1)
    If ModemNum = 0 Then
        MsgBox "HANGUP: " & MWErrorMsg$()
        Exit Sub
    End If
    
    '-- Send the Hangup command and return the result code
    HangUpCmd$ = RemoveCaret$(ZTrim$(MW_CurModem(ModemNum).Hangup))
    
    Call SendModemCmd(Comm1, HangUpCmd$, Attempts, 5, Result)

    '-- Check Result Code
    If Result <> MW_RESULT_OK Then
        MWSetError MW_HANGUP_ERROR
    Else
        MWSetError MW_HANGUP_OK
    End If

HUExit:

    '-- Reset AutoProcess and RThreshold
    If AutoP Then
        Comm1.AutoProcess = AutoP
    End If
    If RT Then
        Comm1.RThreshold = RT
    End If
    
    MW_CurRoutine = MW_PROC_NONE

End Sub

Sub PlaceCall (Comm1 As Control, PhoneNum$, Attempts%, TimeOut%, Result%, ModemBaud&, RealBaud&, Mode$)
'-- Dials a phone number and returns the result.
Dim Test%, AutoP%, RT%, ModemNum%, I%, CDFlag%, ExcludeFlag%
Dim CARRIER_Pos%, CrPos%, CONNECT_Pos%, L%, LenBaud%, AscVal%
Dim S$, DialCmd$, Connect$, Temp$, DialStr$, Received$
Dim BaudStr$

    '-- Test Comm port validity
    On Error Resume Next
    Test = Comm1.CommPort
    If Err Then
        '-- Port can not be accessed
        MWSetError MW_ER_CTRLERROR
        Exit Sub
    Else
        If Comm1.PortOpen = False Then
            '-- Port it closed
            MWSetError MW_ER_PORTCLOSED
            Exit Sub
        End If
    End If
    On Error GoTo 0
    
    '-- Initialize MW_CancelFlag
    MW_CancelFlag = False
    
    '-- Tell everybody where we are
    If Not MW_CurRoutine Then
        MW_CurRoutine = MW_PROC_PLACECALL
    End If
    
    '-- Turn Off AutoProcess & RThreshold
    AutoP = Comm1.AutoProcess
    If AutoP Then
        Comm1.AutoProcess = PDQ_AUTOPROCESS_NONE
    End If
    RT = Comm1.RThreshold
    If RT Then
        Comm1.RThreshold = 0
    End If

    '-- Are we recording a script?
    If MW_ScriptRecording Then
        S$ = Comm1.Settings
        AddToRecScriptData "SETTINGS", UCase$(S$)
        DialCmd$ = Trim$(PhoneNum$)
        If Attempts Then
            DialCmd$ = DialCmd$ & "," & Str$(Attempts)
        End If
        AddToRecScriptData "DIAL", DialCmd$
    End If
    
    '-- Initialize Variables
    ModemBaud& = 0
    RealBaud& = 0
    Mode$ = ""
    Result = 0
    MWSetError 0
    
    '-- Get the current modem index
    ModemNum = GetModemIndex(Comm1)
    If ModemNum = 0 Then
        MsgBox "PLACECALL: " & MWErrorMsg$()
        Exit Sub
    End If
    
    Connect$ = ZTrim$(MW_CurModem(ModemNum).Connect)
    If Len(Connect$) = 0 Then
        Connect$ = "CONNECT"
    End If
    
    '-- Create the dial string
    Temp$ = ZTrim$(MW_CurModem(ModemNum).Dial)
    If Len(Temp$) Then
        DialStr$ = Temp$ & Trim$(PhoneNum$) & MW_CR$
    Else
        DialStr$ = "ATDT" & Trim$(PhoneNum$) & MW_CR$
    End If

    '-- Status print
    MWStatus 0, "Dialing..."
    
    For I = 1 To Attempts
        
        '-- Flush the receive buffer and send the AT command
        Comm1.InBufferCount = 0
        Comm1.Output = DialStr$
    
        Do
        
        '-- Wait for the modem to connect.
        CDFlag = False
        ExcludeFlag = True
        
        '-- Wait for a carriage return/linefeed
        Received$ = WaitFor$(Comm1, MW_CRLF$, TimeOut, CDFlag, ExcludeFlag)
        
        Select Case MWError()
            Case MW_ER_CANCELED    '-- Are we Canceled?
                '-- Send a character to abort the call
                Comm1.Output = " "
                Exit For
            Case MW_ER_TIMEOUT     '-- Did we get something else?
                '-- Send a character to abort the call
                Comm1.Output = " "
                Result = Str2Result(Comm1, Received$)
                Exit For
            Case MW_WAITFOR_OK
                '-- Have we received something from the modem?
                If Len(Received$) Then
                    Result = Str2Result(Comm1, Received$)
                    Select Case Result
                        Case MW_RESULT_CARRIER
                            CARRIER_Pos = InStr(Received$, "CARRIER")
                            If CARRIER_Pos Then
                                BaudStr$ = Mid$(Received$, CARRIER_Pos + 8)
                                CrPos = InStr(BaudStr$, MW_CR$)
                                If CrPos Then
                                    RealBaud& = Val(Left$(BaudStr$, CrPos - 1))
                                Else
                                    RealBaud& = Val(BaudStr$)
                                End If
                            End If
                        Case MW_RESULT_CONNECT
                            CONNECT_Pos = InStr(Received$, Connect$)
                            If Len(Received$) < 8 Then
                                ModemBaud& = 300
                            Else
                                BaudStr$ = Mid$(Received$, CONNECT_Pos + 8)
                                '-- Find position of first non-numeric character
                                L = Len(BaudStr$)
                                For LenBaud = 1 To L
                                    AscVal = Asc(Mid$(BaudStr$, LenBaud, 1))
                                    If AscVal < 48 Or AscVal > 57 Then
                                        Exit For
                                    End If
                                Next
                                '-- Get ModemBaud&
                                If LenBaud > L Then
                                    ModemBaud& = Val(BaudStr$)
                                Else
                                    ModemBaud& = Val(Left$(BaudStr$, LenBaud))
                                    '-- Get the rest as Mode$
                                    Mode$ = Mid$(BaudStr$, LenBaud + 1)
                                End If
                            End If
                            If CARRIER_Pos Then
                                If RealBaud& = 0 Then
                                    RealBaud& = ModemBaud&
                                End If
                            End If
                            MWSetError MW_PLACECALL_OK
                            '-- If 2400 baud, you have to pause for a few
                            '   seconds to let the modem handshaking complete.
                            If ModemBaud& <= 2400 Then
                                MWPause 8
                            End If
                            GoTo PCExit:
                        Case MW_RESULT_RING
                            '-- Just a phone ring, keep waiting
                        Case MW_RESULT_BUSY
                            '-- Busy, Redial
                            MWSetError MW_PLACECALL_OK
                            Exit Do
                        Case MW_RESULT_NO_CARRIER, MW_RESULT_TIMEOUT, MW_RESULT_NO_ANSWER, MW_RESULT_ERROR, MW_RESULT_NO_DIALTONE
                            MWSetError MW_PLACECALL_OK
                            Exit For
                        Case 0
                            '-- No Known Result Code.. Continue waiting
                        Case Else
                            '-- Valid result code, not CONNECT, Continue waiting
                    End Select
                End If
        End Select
        
        DoEvents
        Loop
    
    DoEvents
    Next

PCExit:

    '-- Reset AutoProcess and RThreshold
    If AutoP Then
        Comm1.AutoProcess = AutoP
    End If
    If RT Then
        Comm1.RThreshold = RT
    End If
    
    MW_CurRoutine = MW_PROC_NONE
End Sub

Function Result2Str$ (Comm1 As Control, ResultCode)
'-- Converts a modem result code into a string

Dim ModemNum As Integer
Dim Connect$, Busy$

    '-- Get the current modem index
    ModemNum = GetModemIndex(Comm1)
    If ModemNum = 0 Then
        MsgBox "RESULT2STR: " & MWErrorMsg$()
        Exit Function
    End If
    
    Connect$ = ZTrim$(MW_CurModem(ModemNum).Connect)
    If Len(Connect$) = 0 Then
        Connect$ = "CONNECT"
    End If
    
    Busy$ = ZTrim$(MW_CurModem(ModemNum).Busy)
    If Len(Busy$) = 0 Then
        Busy$ = "BUSY"
    End If

    Select Case ResultCode
        Case MW_RESULT_OK
            Result2Str$ = "OK"
        Case MW_RESULT_CARRIER
            Result2Str$ = "CARRIER"
        Case MW_RESULT_CONNECT
            Result2Str$ = Connect$
        Case MW_RESULT_RING
            Result2Str$ = "RING"
        Case MW_RESULT_NO_CARRIER
            Result2Str$ = "NO CARRIER"
        Case MW_RESULT_ERROR
            Result2Str$ = "ERROR"
        Case MW_RESULT_NO_DIALTONE
            Result2Str$ = "NO DIALTONE"
        Case MW_RESULT_BUSY
            Result2Str$ = Busy$
        Case MW_RESULT_NO_ANSWER
            Result2Str$ = "NO ANSWER"
        Case MW_RESULT_TIMEOUT
            Result2Str$ = "TIMEOUT"
    End Select

End Function

Sub SendModemCmd (Comm1 As Control, Cmd$, Attempts%, TimeOut%, Result%)
'-- Sends a command to your modem. You can use this to send your own
'   initialization strings or setup strings to your modem. Returns
'   the result code.

Dim Test As Integer, AutoP As Integer, RT As Integer, Index As Integer, OldRoutine As Integer
Dim I As Integer, CDFlag As Integer, ExcludeFlag As Integer, Found As Integer
Dim Attention$, ResultStr$

    '-- Test Comm port validity
    On Error Resume Next
    Test = Comm1.CommPort
    If Err Then
        '-- Port can not be accessed
        MWSetError MW_ER_CTRLERROR
        Exit Sub
    Else
        If Comm1.PortOpen = False Then
            '-- Port it closed
            MWSetError MW_ER_PORTCLOSED
            Exit Sub
        End If
    End If
    On Error GoTo 0

    '-- InitModemWare initializes MW_CR$, MW_LF$, etc.
    If MW_Init = 0 Then Call InitModemWare

    '-- Tell everybody where we are
    OldRoutine = MW_CurRoutine
    MW_CurRoutine = MW_PROC_SENDMODEMCMD

    '-- Initialize MW_CancelFlag
    MW_CancelFlag = False

    '-- Initialize variables
    Result = 0
    MWSetError 0

    '-- Turn Off AutoProcess & RThreshold
    AutoP = Comm1.AutoProcess
    If AutoP Then
        Comm1.AutoProcess = PDQ_AUTOPROCESS_NONE
    End If
    RT = Comm1.RThreshold
    If RT Then
        Comm1.RThreshold = 0
    End If

    '-- Create an array for WaitForA$
    ReDim WaitArray$(1 To 2)
    WaitArray$(1) = MW_CRLF$
    WaitArray$(2) = MW_CR$
    
    '-- Get the current modem index
    Index = GetModemIndex(Comm1)
    If Index = 0 Then
        MsgBox "SENDMODEMCMD: " & MWErrorMsg$()
        Exit Sub
    End If
    
    '-- Make sure that the command is semi-valid
    If Len(Cmd$) = 0 Then
        MWSetError MW_SENDMODEMCMD_NOCMD
        GoTo SMCExit
    Else
        Attention$ = RemoveCaret$(ZTrim$(MW_CurModem(Index).Attention))
        '-- Starts with attention string?
        If UCase$(Left$(Cmd$, 2)) <> Attention$ Then
            Cmd$ = Attention$ & Cmd$
        End If
        '-- Ends in CR?
        If Right$(Cmd$, 1) <> MW_CR$ Then
            Cmd$ = Cmd$ & MW_CR$
        End If
        '-- Starts with a quote?
        If Left$(Cmd$, 1) = Chr$(34) Then
            Cmd$ = Mid$(Cmd$, 2)
        End If
    End If

    '-- Are we recording a script?
    If MW_ScriptRecording Then
        AddToRecScriptData "SEND", Cmd$
    End If
    
    '-- Flush the receive buffer
    Comm1.InBufferCount = 0

    If Attempts = 0 Then Attempts = 1

    For I = 1 To Attempts
        '-- Send the command
        Comm1.Output = Cmd$
        '-- Wait for the result code
        Do
            CDFlag = False
            ExcludeFlag = False
            ResultStr$ = WaitForA$(Comm1, WaitArray$(), TimeOut, CDFlag, ExcludeFlag, Found)
            Select Case MWError()
                Case MW_ER_TIMEOUT, MW_ER_CANCELED
                    GoTo SMCExit
                Case MW_WAITFOR_OK
                    If Len(ResultStr$) Then
                        '-- Get the result code
                        Result = Str2Result(Comm1, ResultStr$)
                        If Result Then
                            '-- Got a legitimate result
                            
                            Exit For
                        End If
                    End If
            End Select
        Loop
        '-- Pause 1 second
        MWPause 1
    Next

    '-- Return OK
    MWSetError MW_SENDMODEMCMD_OK

SMCExit:

    '-- Reset AutoProcess and RThreshold
    If AutoP Then
        Comm1.AutoProcess = AutoP
    End If
    If RT Then
        Comm1.RThreshold = RT
    End If
    
    MW_CurRoutine = OldRoutine

End Sub

Sub SetModemSettings (Comm1 As Control)
'-- Lets the user set up their comm ports for ModemWare applications.

Dim PortWasOpen As Integer, AutoP As Integer
Dim T$, OldSetting$
    
    '-- Make sure the port is closed
    If Comm1.PortOpen Then
        PortWasOpen = True
        OldSetting$ = Comm1.Settings
        Comm1.PortOpen = False
    End If
    
    '-- MW_Global_Comm1 is a global object that can be accessed by
    '   the Modem setup forms.
    Set MW_Global_Comm1 = Comm1
    
    '-- Tell everybody where we are
    If Not MW_CurRoutine Then MW_CurRoutine = MW_PROC_SETMODEMSETTINGS
    
    '-- Set The Caption
    MWModem.Caption = Comm1.Tag & " Modem Settings"

    '-- Show the modem configuration window
    MWModem.Show 1
    
    '-- Was the cancel button pressed?
    T$ = MWModem.Tag
    If Len(T$) = 0 Then
        '-- Turn On AutoProcess
        If AutoP Then Comm1.AutoProcess = AutoP
        Exit Sub
    End If
    
    '-- Re-open the port if it was opened.
    If PortWasOpen Then
        Comm1.Settings = OldSetting$
        Comm1.PortOpen = True
        PortWasOpen = GetModemIndex(Comm1)
        Call SendModemCmd(Comm1, MW_CurModem(PortWasOpen).Attention, 1, 1, PortWasOpen)
    End If

End Sub

Sub LoadModemSettings (Comm1 As Control)
    
Dim Test As Integer, AutoP As Integer, RT As Integer, Dummy As Integer
Dim NumModems As Integer, L As Integer, TimeOut As Integer, C As Integer
Dim Result As Integer, ModemIndex As Integer
Dim ConfigFile$, AppName$, Temp$, T$, InitString$, NextCmd$
Dim Nam$, Num&
Static ErrFlag
    
    '-- Test Comm port validity
    On Error Resume Next
    Test = Comm1.PortOpen
    If Err Then
        '-- Port can not be accessed
        MWSetError MW_ER_CTRLERROR
        Exit Sub
    End If
    On Error GoTo 0
    
    '-- Turn Off AutoProcess and RThreshold
    AutoP = Comm1.AutoProcess
    If AutoP Then
        Comm1.AutoProcess = PDQ_AUTOPROCESS_NONE
    End If
    RT = Comm1.RThreshold
    If RT Then
        Comm1.RThreshold = 0
    End If
    
    '-- Initialize return variables
    MWSetError 0
    
    '-- Is the modem is currently connected.
    If Comm1.CDHolding Then
        MWSetError MW_INITMODEM_ONLINE
        Exit Sub
    End If

    '-- InitModemWare initializes MW_CR$, MW_LF$, etc.
    If MW_Init = False Then Call InitModemWare
    
    '-- Get the configuration INI filename
    ConfigFile$ = GetConfigFileName$()
    
    '-- All Modem Settings are listed under "[Modem]" in the INI file.
    AppName$ = "Modem_" & Comm1.Tag
        
    '-- Get Modem Number
    Temp$ = Space$(10)
    Dummy = MWGetConfigString(AppName$, "ModemNumber", "", Temp$, 10, ConfigFile$)
    Num& = Val(Temp$)
    
    '-- Does the configuration exist?
    If Num& = 0 Then
        Call SetModemSettings(Comm1)
    Else
    '-- Get Modem Number
        On Error Resume Next
        NumModems = UBound(MW_CurModem)
        On Error GoTo 0
        NumModems = NumModems + 1
        ReDim Preserve MW_CurModem(1 To NumModems) As MW_ModemInfoType
        ReDim Preserve MW_CurModemIndex(1 To NumModems) As String

        MW_CurModem(NumModems).ModemNumber = Num&
        T$ = Comm1.Tag
        
        '-- Do a little programmer brain testing.
        If NumModems > 1 Then
            If Len(T$) = 0 Then
                MsgBox "You have not set the Tag property of this comm control. ModemWare must identify each comm control by its Tag property. Please set the Tag property of each comm control in your project to a unique string."
                Exit Sub
            End If
        End If
        
        MW_CurModemIndex(NumModems) = Comm1.Tag

        '-- Get Modem Name
        Temp$ = Space$(128)
        L = MWGetConfigString(AppName$, "ModemName", "", Temp$, 128, ConfigFile$)
        If L Then
            MW_CurModem(NumModems).ModemName = Left$(Temp$, L)
        Else
            MW_CurModem(NumModems).ModemName = "Generic 2400 Baud Modem"
        End If

        '-- Get Highest Baud rate
        Temp$ = Space$(10)
        L = MWGetConfigString(AppName$, "HighestBaud", "", Temp$, 10, ConfigFile$)
        If L Then
            MW_CurModem(NumModems).HighestBaud = Val(Temp$)
        Else
            MW_CurModem(NumModems).HighestBaud = 2400
        End If

        '-- Get Init String
        Temp$ = Space$(128)
        L = MWGetConfigString(AppName$, "InitString", "", Temp$, 128, ConfigFile$)
        If L Then
            MW_CurModem(NumModems).InitString = Left$(Temp$, L)
        Else
            MW_CurModem(NumModems).InitString = "ATZ^M"
        End If

        '-- Get Connect String
        Temp$ = Space$(128)
        L = MWGetConfigString(AppName$, "Connect", "", Temp$, 128, ConfigFile$)
        If L Then
            MW_CurModem(NumModems).Connect = Left$(Temp$, L)
        Else
            MW_CurModem(NumModems).Connect = "CONNECT"
        End If
        
        '-- Get Attention String
        Temp$ = Space$(24)
        L = MWGetConfigString(AppName$, "Attention", "", Temp$, 24, ConfigFile$)
        If L Then
            MW_CurModem(NumModems).Attention = Left$(Temp$, L)
        Else
            MW_CurModem(NumModems).Attention = "AT"
        End If
        
        '-- Get Hangup String
        Temp$ = Space$(24)
        L = MWGetConfigString(AppName$, "Hangup", "", Temp$, 24, ConfigFile$)
        If L Then
            MW_CurModem(NumModems).Hangup = Left$(Temp$, L)
        Else
            MW_CurModem(NumModems).Hangup = "ATH^M"
        End If
        
        '-- Get Reset String
        Temp$ = Space$(24)
        L = MWGetConfigString(AppName$, "Reset", "", Temp$, 24, ConfigFile$)
        If L Then
            MW_CurModem(NumModems).Reset = Left$(Temp$, L)
        Else
            MW_CurModem(NumModems).Reset = "ATZ^M"
        End If
        
        '-- Get Answer String
        Temp$ = Space$(24)
        L = MWGetConfigString(AppName$, "Answer", "", Temp$, 24, ConfigFile$)
        If L Then
            MW_CurModem(NumModems).Answer = Left$(Temp$, L)
        Else
            MW_CurModem(NumModems).Answer = "ATA^M"
        End If
        
        '-- Get Dial String
        Temp$ = Space$(24)
        L = MWGetConfigString(AppName$, "Dial", "", Temp$, 24, ConfigFile$)
        If L Then
            MW_CurModem(NumModems).Dial = Left$(Temp$, L)
        Else
            MW_CurModem(NumModems).Dial = "ATDT"
        End If
        
        '-- Get Busy String
        Temp$ = Space$(24)
        L = MWGetConfigString(AppName$, "Busy", "", Temp$, 24, ConfigFile$)
        If L Then
            MW_CurModem(NumModems).Busy = Left$(Temp$, L)
        Else
            MW_CurModem(NumModems).Busy = "BUSY"
        End If
    End If

    GoTo LoadModemInit:


LoadModemSet:       '-- Let the user pick his/her modem.
    
    Call SetModemSettings(Comm1)

LoadModemInit:      '-- Initialize the Modem

    '-- Set up for invalid comm port processing
    On Error Resume Next
    NumModems = GetModemIndex(Comm1)
    '-- Try to open the port
    If Comm1.PortOpen = False Then
        Comm1.PortOpen = True
    End If
        
    '-- Initialize the modem
    Call MWStatus(0, "Initializing Modem...")
    Call LoadPortSettings(Comm1)
    ErrFlag = False
    TimeOut = 10
    Screen.MousePointer = 11
    If NumModems = 0 Then
        Temp$ = Space$(10)
        Dummy = MWGetConfigString(AppName$, "ModemNumber", "", Temp$, 10, ConfigFile$)
        NumModems = Val(Temp$)
    End If
    InitString$ = ZTrim$(MW_CurModem(NumModems).InitString)
    Temp$ = RemoveCaret$(InitString$)

    Do

        C = InStr(Temp$, MW_CR$)
        If C Then
            NextCmd$ = ZTrim$(Left$(Temp$, C))
            If Len(Temp$) > C Then
                Temp$ = Mid$(Temp$, C + 1)
            End If
        Else
            Exit Do
        End If

MWTryAgain:
            
        Call SendModemCmd(Comm1, NextCmd$, 1, TimeOut, Result)
        Select Case Result
            Case MW_RESULT_OK
                Exit Do
            '-- Continue with the loop
            Case MW_RESULT_ERROR, MW_RESULT_TIMEOUT
                If ErrFlag = False Then
                    TimeOut = TimeOut + 10
                    ErrFlag = True
                    GoTo MWTryAgain:
                Else
                    Screen.MousePointer = 0
                    MsgBox "Modem Setup Error: Please re-configure your modem."
                    If AutoP Then Comm1.AutoProcess = AutoP
                    Exit Sub
                End If
            Case Else
                Screen.MousePointer = 0
                Call MWStatus(0, MWErrorMsg$())
                If AutoP Then Comm1.AutoProcess = AutoP
                Exit Sub
        End Select
        
        MWPause 3
    
    Loop

    Screen.MousePointer = 0
    ModemIndex = GetModemIndex(Comm1)
    Nam$ = ZTrim$(MW_CurModem(ModemIndex).ModemName)
    Call MWStatus(0, "Online: " & Nam$)

LMSBail:

    '-- Turn On AutoProcess
    If AutoP Then
        Comm1.AutoProcess = AutoP
    End If
    If RT Then
        Comm1.RThreshold = RT
    End If
    
End Sub

Function Str2Result (Comm1 As Control, ResultStr$)
'-- Internal: This function converts a result code string to
'   it's code number

Dim ModemNum As Integer, A As Integer, B As Integer, C As Integer, D As Integer
Dim E As Integer, F As Integer, G As Integer, H As Integer, I As Integer, J As Integer
Dim Busy$, Connect$

    '-- Get the current modem index
    ModemNum = GetModemIndex(Comm1)
    If ModemNum = 0 Then
        MsgBox "STR2RESULT: " & MWErrorMsg$()
        Exit Function
    End If
    
    Busy$ = ZTrim$(MW_CurModem(ModemNum).Busy)
    If Len(Busy$) = 0 Then
        Busy$ = "BUSY"
    End If

    Connect$ = ZTrim$(MW_CurModem(ModemNum).Connect)
    If Len(Connect$) = 0 Then
        Connect$ = "CONNECT"
    End If

    A = InStr(ResultStr$, "OK")
    B = InStr(ResultStr$, Connect$)
    C = InStr(ResultStr$, "RING")
    D = InStr(ResultStr$, "NO CARRIER")
    E = InStr(ResultStr$, "ERROR")
    F = InStr(ResultStr$, "NO DIALTONE")
    G = InStr(ResultStr$, Busy$)
    H = InStr(ResultStr$, "NO ANSWER")
    I = InStr(ResultStr$, "TIMEOUT")
    J = InStr(ResultStr$, "CARRIER")

    If A Then
        Str2Result = MW_RESULT_OK
    ElseIf B Then
        Str2Result = MW_RESULT_CONNECT
    ElseIf C Then
        Str2Result = MW_RESULT_RING
    ElseIf D Then
        Str2Result = MW_RESULT_NO_CARRIER
    ElseIf E Then
        Str2Result = MW_RESULT_ERROR
    ElseIf F Then
        Str2Result = MW_RESULT_NO_DIALTONE
    ElseIf G Then
        Str2Result = MW_RESULT_BUSY
    ElseIf H Then
        Str2Result = MW_RESULT_NO_ANSWER
    ElseIf I Then
        Str2Result = MW_RESULT_TIMEOUT
    ElseIf J Then
        Str2Result = MW_RESULT_CARRIER
    End If

End Function

Sub WaitForCall (Comm1 As Control, NumRings, ModemBaud&, RealBaud&, Mode$)
'-- Waits for a call and connects the modem. This is host mode made easy.
'   Just call this routine, and it will return when a caller has connected.
Dim Test%, ModemNum%, AutoP%, RT%, TimeOut%, NumTries%, Result%, CDFlag%
Dim ExcludeFlag%, CARRIER_Pos%, CONNECT_Pos%, L%, LenBaud%, AscVal%
Dim Connect$, Attention$, InitString$, Received$, BaudStr$

    '-- Test Comm port validity
    On Error Resume Next
    Test = Comm1.CommPort
    If Err Then
        '-- Port can not be accessed
        MWSetError MW_ER_CTRLERROR
        Exit Sub
    Else
        If Comm1.PortOpen = False Then
            '-- Port it closed
            MWSetError MW_ER_PORTCLOSED
            Exit Sub
        End If
    End If
    On Error GoTo 0
    
    '-- InitModemWare initializes MW_CR$, MW_LF$, etc.
    Call InitModemWare
    
    '-- Initialize MW_CancelFlag
    MW_CancelFlag = False
    
    '-- Tell everybody where we are
    If Not MW_CurRoutine Then
        MW_CurRoutine = MW_PROC_WAITFORCALL
    End If
    
    '-- Initialize and validate variables
    ModemBaud& = 0
    RealBaud& = 0
    Mode$ = ""
    MWSetError 0
    If NumRings = 0 Then
        NumRings = 1
    End If
    
    '-- Get the current modem index
    ModemNum = GetModemIndex(Comm1)
    If ModemNum = 0 Then
        MsgBox "WAITFORCALL: " & MWErrorMsg$()
        Exit Sub
    End If
    
    Connect$ = ZTrim$(MW_CurModem(ModemNum).Connect)
    If Len(Connect$) = 0 Then
        Connect$ = "CONNECT"
    End If
    
    '-- Turn Off AutoProcess & RThreshold
    AutoP = Comm1.AutoProcess
    If AutoP Then
        Comm1.AutoProcess = PDQ_AUTOPROCESS_NONE
    End If
    RT = Comm1.RThreshold
    If RT Then
        Comm1.RThreshold = 0
    End If

    '-- Are we recording a script?
    If MW_ScriptRecording Then
        AddToRecScriptData "WAITFORCALL", ""
    End If
    
    '-- Initialize the modem and wait up to 5 seconds for an OK
    TimeOut = 5
    NumTries = 3
    Attention$ = RemoveCaret$(ZTrim$(MW_CurModem(ModemNum).Attention))
    InitString$ = Attention$ & "S0=" & LTrim$(Str$(NumRings)) & MW_CR$
    Call SendModemCmd(Comm1, InitString$, NumTries, TimeOut, Result)
    
    Select Case MWError()
        '-- Are we Canceled or timed out?
        Case MW_ER_CANCELED, MW_ER_TIMEOUT
            GoTo WFCExit:
    End Select
    
    '-- Status Print
    MWStatus 0, "Waiting For a Call..."

    '-- Wait for the modem to connect.
    Do
        
        TimeOut = 3600
        CDFlag = False
        ExcludeFlag = True
        Received$ = WaitFor$(Comm1, MW_CR$, TimeOut, CDFlag, ExcludeFlag)
        
        Select Case MWError()
            Case MW_ER_CANCELED    '-- Are we Canceled?
                '-- Send an ATS0=0
                Call SendModemCmd(Comm1, Attention$ & "S0=0", 2, 10, Result)
                MWSetError MW_ER_CANCELED
                GoTo WFCExit:
            Case MW_ER_TIMEOUT      '-- Did we get something else?
                Result = Str2Result(Comm1, Received$)
                GoTo WFCExit:
            Case MW_WAITFOR_OK
                '-- Have we received something from the modem?
                If Len(Received$) Then
                    Result = Str2Result(Comm1, Received$)
                    Select Case Result
                        Case MW_RESULT_CARRIER
                            CARRIER_Pos = InStr(Received$, "CARRIER")
                            If CARRIER_Pos Then
                                BaudStr$ = Mid$(Received$, CARRIER_Pos + 8)
                                RealBaud& = Val(BaudStr$)
                            End If
                        Case MW_RESULT_CONNECT
                            CONNECT_Pos = InStr(Received$, Connect$)
                            If Len(Received$) < 8 Then
                                ModemBaud& = 300
                            Else
                                BaudStr$ = Mid$(Received$, CONNECT_Pos + 8)
                                '-- Find position of first non-numeric character
                                L = Len(BaudStr$)
                                For LenBaud = 1 To L
                                    AscVal = Asc(Mid$(BaudStr$, LenBaud, 1))
                                    If AscVal < 48 Or AscVal > 57 Then
                                        Exit For
                                    End If
                                Next
                                '-- Get ModemBaud&
                                If LenBaud > L Then
                                    ModemBaud& = Val(BaudStr$)
                                Else
                                    ModemBaud& = Val(Left$(BaudStr$, LenBaud))
                                    '-- Get the rest as Mode$
                                    Mode$ = Mid$(BaudStr$, LenBaud + 1)
                                End If
                            End If
                            If CARRIER_Pos Then
                                If RealBaud& = 0 Then
                                    RealBaud& = ModemBaud&
                                End If
                            End If
                            MWSetError MW_WAITFORCALL_OK
                            '-- If 2400 baud, you have to pause for a few
                            '   seconds to let the modem handshaking complete.
                            If ModemBaud& <= 2400 Then
                                MWPause 9
                            End If
                            Exit Do
                        Case MW_RESULT_RING
                            '-- Just a phone ring, keep waiting
                            MWStatus 0, "Incomming Call... Answering"
                        Case MW_RESULT_BUSY
                            '-- Busy.
                            MWSetError MW_WAITFORCALL_OK
                            Exit Do
                        Case MW_RESULT_ERROR
                            MWSetError MW_WAITFORCALL_OK
                            Exit Do
                        Case 0
                            '-- No Known Result Code.. Continue waiting
                        Case Else
                            '-- Valid result code, not CONNECT, Continue waiting
                    End Select
                End If
        End Select
    Loop
    
WFCExit:

    '-- Reset AutoProcess and RThreshold
    If AutoP Then
        Comm1.AutoProcess = AutoP
    End If
    If RT Then
        Comm1.RThreshold = RT
    End If
    
    MW_CurRoutine = MW_PROC_NONE
    
End Sub

