Option Explicit

'-- ModemWare Copyright (c) 1992 - 1994 Crescent Software.
'   Written by Carl Franklin with help from and thanks to Bill Conley.

Sub AddToRecScriptData (Cmd$, Dat$)
'-- This routine is called by all modemware routines
'   that wish to log action to the script. If you have custom programmed
'   actions that you wish to log, you can do so by calling this routine.
'   The only restriction is that Cmd$ must be a valid script command.
Dim NewScript%

    NewScript = UBound(MW_RecScriptData) + 1
    ReDim Preserve MW_RecScriptData(1 To NewScript) As MW_ScriptDataType
    MW_RecScriptData(NewScript).Cmd = Cmd$
    MW_RecScriptData(NewScript).Dat = Dat$

End Sub

Function Script2Cmd (NextLine$)
'-- Internal function that converts a script command (string) to
'   a script command code.

Dim Sp As Integer, T As Integer
Dim Temp$, Cmd$

    Temp$ = UCase$(NextLine$)
    Sp = InStr(Temp$, " ")
    If Sp Then
        Cmd$ = Left$(Temp$, Sp - 1)
    Else
        T = InStr(Temp$, Chr$(9))
        If T > 1 Then
            Cmd$ = Left$(Temp$, T - 1)
        Else
            If Len(NextLine$) Then
                Cmd$ = NextLine$
            End If
        End If
    End If

    Select Case Cmd$
        Case "PORT"
            Script2Cmd = MW_SCRIPT_CMD_PORT
        Case "SETTINGS"
            Script2Cmd = MW_SCRIPT_CMD_SETTINGS
        Case "DIAL"
            Script2Cmd = MW_SCRIPT_CMD_DIAL
        Case "TIMEOUT"
            Script2Cmd = MW_SCRIPT_CMD_TIMEOUT
        Case "WAITFOR"
            Script2Cmd = MW_SCRIPT_CMD_WAITFOR
        Case "SEND"
            Script2Cmd = MW_SCRIPT_CMD_SEND
        Case "HANGUP"
            Script2Cmd = MW_SCRIPT_CMD_HANGUP
        Case "PROTOCOL"
            Script2Cmd = MW_SCRIPT_CMD_PROTOCOL
        Case "UPLOAD"
            Script2Cmd = MW_SCRIPT_CMD_UPLOAD
        Case "DOWNLOAD"
            Script2Cmd = MW_SCRIPT_CMD_DOWNLOAD
        Case "PAUSE"
            Script2Cmd = MW_SCRIPT_CMD_PAUSE
        Case "STOP"
            Script2Cmd = MW_SCRIPT_CMD_STOP
        Case "END"
            Script2Cmd = MW_SCRIPT_CMD_END
        Case "INPUT"
            Script2Cmd = MW_SCRIPT_CMD_INPUT
        Case "CAPTURE"
            Script2Cmd = MW_SCRIPT_CMD_CAPTURE
        Case "CLOSECAPTURE"
            Script2Cmd = MW_SCRIPT_CMD_CLOSECAPTURE
        Case "ON_TIMEOUT_GOTO"
            Script2Cmd = MW_SCRIPT_CMD_ONTIMEOUTGOTO
        
    End Select

End Function

Sub ScriptPlay (Comm1 As Control, FileName$, ErrorLineNum, ErrorLine$)
'-- Plays a ModemWare script file which is simply a text file
'   that contains one or more script commands. For a complete
'   description of the script commands consult the ModemWare documentation.

Dim Test As Integer, FileNum As Integer, AutoP As Integer, RT As Integer
Dim CurLine As Integer, LastLine As Integer, TotalLines As Integer, TotalLabels As Integer
Dim Comment As Integer, ScriptCommand As Integer, PortNum As Integer, OldPort As Integer
Dim WasOpen As Integer, PortSet As Integer, MW_CaptureFile As Integer, C As Integer
Dim Index As Integer, S As Integer, Retries As Integer, TimeOut As Integer
Dim Result As Integer, CDFlag As Integer, ExcludeFlag As Integer, OldLen As Integer
Dim ProtocolSetFlag As Integer, E As Integer, T As Integer, I As Integer, TimeoutLine As Integer
Dim RealBaud&, ModemBaud&, NumSecs!
Dim Temp$, NextLine$, TimeoutLabel$, Settings$, OldSet$, CapFileName$, In$, Argument$
Dim DisplayStr$, Variable$, Num$, Mode$, Msg$, Wait$, Received$, OutString$, Prtcl$
Dim XFileName$

    '-- Test Comm port validity
    On Error Resume Next
    Test = Comm1.CommPort
    If Err Then
        '-- Port can not be accessed
        MWSetError MW_ER_CTRLERROR
        Exit Sub
    End If
    On Error GoTo 0

    Set MW_Global_Comm1 = Comm1

    '-- InitModemWare initializes MW_CR$, MW_LF$, etc.
    Call InitModemWare
    
    '-- Initialize return variables
    MWSetError 0
    
    '-- Open the script file, testing it's validity
    FileNum = FreeFile
    On Error Resume Next
    Open FileName$ For Input As FileNum
    If Err Then
        MWSetError MW_SCRIPTPLAY_BAD_FILE
        Exit Sub
    End If
    On Error GoTo 0

    '-- Turn Off AutoProcess & RThreshold
    AutoP = Comm1.AutoProcess
    If AutoP Then
        Comm1.AutoProcess = PDQ_AUTOPROCESS_NONE
    End If
    RT = Comm1.RThreshold
    If RT Then
        Comm1.RThreshold = 0
    End If

    '-- Read the script into an array
    CurLine = 1
    Do
        Line Input #FileNum, Temp$
        If EOF(FileNum) Then
            LastLine = True
            If Len(Temp$) = 0 Then
                TotalLines = CurLine - 1
                Exit Do
            End If
        End If
        ReDim Preserve script$(1 To CurLine)
        script$(CurLine) = Trim$(Temp$)
        '-- Label?
        If Left$(Temp$, 1) = ":" Then
            '-- Add the label to the Labels array
            On Error Resume Next
            TotalLabels = UBound(MW_ScriptLabel)
            TotalLabels = TotalLabels + 1
            On Error GoTo 0
            ReDim Preserve MW_ScriptLabel(1 To TotalLabels) As MW_ScriptLabelType
            MW_ScriptLabel(TotalLabels).Label = Temp$
            MW_ScriptLabel(TotalLabels).LineNum = CurLine
        End If
        If LastLine Then
            TotalLines = CurLine
            Exit Do
        Else
            CurLine = CurLine + 1
        End If
        DoEvents
    Loop
    Close FileNum
    
    '-- Main loop
    For ErrorLineNum = 1 To TotalLines

        '-- Read the next line
        NextLine$ = script$(ErrorLineNum)
        ErrorLine$ = NextLine$

        '-- Remove comments (if any)
        Comment = InStr(NextLine$, ";") '-- Semicolon
        If Comment > 1 Then
            NextLine$ = Left$(NextLine$, Comment - 1)
        ElseIf Comment = 1 Then
            GoTo PS_LoopHere
        End If
        Comment = InStr(NextLine$, "'") '-- Apostrophe
        If Comment > 1 Then
            NextLine$ = Left$(NextLine$, Comment - 1)
        ElseIf Comment = 1 Then
            GoTo PS_LoopHere
        End If

        '-- Blank line?
        If Len(NextLine$) = 0 Then GoTo PS_LoopHere:

        '-- Label?
        If Left$(NextLine$, 1) = ":" Then GoTo PS_LoopHere:
        
        '-- Process the command
        ScriptCommand = Script2Cmd(NextLine$)
        Select Case ScriptCommand
            Case MW_SCRIPT_CMD_ONTIMEOUTGOTO '--------------------- On Timeout Goto a certian Label.
                If Len(NextLine$) > 15 Then
                    TimeoutLabel$ = Trim$(Mid$(NextLine$, 16))
                    If Left$(TimeoutLabel$, 1) <> ":" Then
                        TimeoutLabel$ = ":" & TimeoutLabel$
                    End If
                End If
            Case MW_SCRIPT_CMD_END '------------------------------- End the script.
                Beep
                MWSetError MW_SCRIPTPLAY_OK
                Exit For
            Case MW_SCRIPT_CMD_PORT '------------------------------ Set the Comm Port Number
                If Len(NextLine$) > 4 Then
                    PortNum = Val(Mid$(NextLine$, 5))
                    '-- Was a port specified?
                    If PortNum Then
                        '-- Yes. Send a status message
                        Call MWStatus(MW_STAT_SCRIPT_INITIALIZING, "")
                        OldPort = Comm1.CommPort
                        '--- Are we already using that port?
                        If PortNum <> OldPort Then
                            '-- No. Is the port open?
                            If Comm1.PortOpen = True Then
                                '-- Yes, close it.
                                WasOpen = True
                                Comm1.PortOpen = False
                            End If
                            '-- Set the new port number
                            Comm1.CommPort = PortNum
                            On Error Resume Next
                            '-- Open the port
                            Comm1.PortOpen = True
                            '-- Is the port valid?
                            If Err Then
                                '-- No. Reset it to the old port
                                MWSetError MW_SCRIPTPLAY_BAD_PORT
                                Comm1.CommPort = OldPort
                                '-- Reopen it if it was open
                                If WasOpen Then
                                    Comm1.PortOpen = True
                                End If
                                '-- Bail
                                Exit For
                            End If
                            On Error GoTo 0
                        Else
                            '-- Yes, we are already using the specified port.
                            '-- Is the port open?
                            If Comm1.PortOpen = False Then
                                '-- No. Open it.
                                On Error Resume Next
                                '-- Open the port
                                Comm1.PortOpen = True
                                '-- Is the port valid?
                                If Err Then
                                    '-- No. Set the error code
                                    MWSetError MW_SCRIPTPLAY_BAD_PORT
                                    '-- Bail
                                    Exit For
                                End If
                            End If
                        End If
                        PortSet = True
                    Else
                        '-- No, a port was not specified
                        MWSetError MW_SCRIPTPLAY_BAD_PORT
                        Exit For
                    End If
                End If
            Case MW_SCRIPT_CMD_SETTINGS '-------------------------- Change the Port's Settings.
                If Len(NextLine$) > 8 Then
                    '-- Tell the world we're initializing.
                    Call MWStatus(MW_STAT_SCRIPT_INITIALIZING, "")
                    '-- Get the settings
                    Settings$ = Mid$(NextLine$, 9)
                    Settings$ = ScriptProcessData$(Settings$)
                    '-- Get the current settings
                    OldSet$ = Comm1.Settings
                    '-- Are they the same as the specified settings?
                    If OldSet$ <> Settings$ Then
                        '-- No. Set the settings
                        On Error Resume Next
                        Comm1.Settings = Settings$
                        If Err Then
                            '-- Oops! Bad settings specified
                            MWSetError MW_SCRIPTPLAY_BAD_SETTINGS
                            '-- Set the port back the way it was
                            Comm1.Settings = OldSet$
                            GoTo PSExitHere:
                        End If
                        On Error GoTo 0
                    End If
                End If
                '-- Status = Port has been set
                Call MWStatus(MW_STAT_SCRIPT_PORTSET, Settings$)
            Case MW_SCRIPT_CMD_CAPTURE '--------------------------- Open a capture file.
                '-- Is the port open?
                If Comm1.PortOpen = False Then
                    MWSetError MW_ER_PORTCLOSED
                    GoTo PSExitHere
                End If
                If Len(NextLine$) > 7 Then
                    '-- Get the Capture filename
                    CapFileName$ = Mid$(NextLine$, 8)
                    CapFileName$ = ScriptProcessData$(CapFileName$)
                    If Len(CapFileName$) = 0 Then
                        MWSetError MW_SCRIPTPLAY_BAD_COMMAND
                        Exit For
                    End If
                    '-- Status = Opening capture file
                    Call MWStatus(MW_STAT_SCRIPT_CAPTURING, CapFileName$)
                    MW_CaptureFile = FreeFile
                    Open CapFileName$ For Output As MW_CaptureFile
                End If
            Case MW_SCRIPT_CMD_CLOSECAPTURE '---------------------- Close the capture file.
                If MW_CaptureFile Then
                    If Comm1.InBufferCount Then
                        In$ = Comm1.Input
                        If MW_DisplayFlag Then
                            If Comm1.Emulation Then
                                Comm1.Disp = In$
                            End If
                        End If
                        Print #MW_CaptureFile, In$
                    End If
                    Close MW_CaptureFile
                    MW_CaptureFile = 0
                End If
            Case MW_SCRIPT_CMD_INPUT '----------------------------- Save user input into a variable.
                '-- Status = Requesting input
                Call MWStatus(MW_STAT_SCRIPT_INPUT, "")
                '-- Is this a valid Input command?
                If Len(NextLine$) > 5 Then
                    Argument$ = Mid$(NextLine$, 6)
                    '-- Is there a comma?
                    C = InStr(Argument$, ",")
                    If C Then
                        If C + 1 < Len(Argument$) Then
                            '-- DisplayStr$ is the text to display
                            Temp$ = Trim$(Left$(Argument$, C - 1))
                            DisplayStr$ = ScriptProcessData$(Temp$)
                            '-- Variable$ is the variable to save the input to.
                            Temp$ = Trim$(Mid$(Argument$, C + 1))
                            Variable$ = ScriptProcessData$(Temp$)
                            '-- If the variable was saved, it comes back as "XXX"
                            '   followed by the index into the MW_ScriptVars array.
                            '   MW_ScriptVars contains the variable name and it's value
                            If Left$(Variable$, 3) = "XXX" Then
                                '-- OK. Get the value with InputBox$
                                Index = Val(Mid$(Variable$, 4))
                                MW_ScriptVars(Index).Value = UCase$(InputBox$(DisplayStr$, UCase$(FileName$))) & MW_CR$
                            Else
                                '-- The variable may have had quotes around it. Not good.
                                MWSetError MW_SCRIPTPLAY_BAD_COMMAND
                                Exit For
                            End If
                        Else
                            '-- Comma, but no variable. Bad Command
                            MWSetError MW_SCRIPTPLAY_BAD_COMMAND
                            Exit For
                        End If
                    Else
                        '-- No comma, Bad Command
                        MWSetError MW_SCRIPTPLAY_BAD_COMMAND
                        Exit For
                    End If
                Else
                    '-- Input was specified, but nothing else!
                    MWSetError MW_SCRIPTPLAY_BAD_COMMAND
                    Exit For
                End If
            Case MW_SCRIPT_CMD_DIAL '------------------------------ Dial a phone number
                '-- Is the port open?
                If Comm1.PortOpen = False Then
                    MWSetError MW_ER_PORTCLOSED
                    GoTo PSExitHere
                End If
                '-- Place call?
                If Len(NextLine$) > 4 Then
                    Temp$ = ZTrim$(Mid$(NextLine$, 5))
                    S = InStr(Temp$, " ")
                    If S Then
                        Num$ = Left$(Temp$, S - 1)
                        Retries = Val(Mid$(Temp$, S + 1))
                    Else
                        Num$ = Temp$
                        Retries = 1
                    End If
                    Num$ = ScriptProcessData$(Num$)
                    '-- Status = Dialing
                    Call MWStatus(MW_STAT_SCRIPT_DIALING, Num$)
                    Call PlaceCall(Comm1, Num$, Retries, TimeOut, Result, ModemBaud&, RealBaud&, Mode$)
                    Select Case Result
                        Case MW_PLACECALL_OK
                            Msg$ = "Connected at" & Str$(ModemBaud&) & " Baud"
                        Case MW_RESULT_NO_CARRIER, MW_RESULT_TIMEOUT
                            Msg$ = "There is no answer"
                            GoSub ProcessTimeout
                        Case MW_RESULT_NO_DIALTONE
                            Msg$ = "No Dial Tone"
                            GoSub ProcessTimeout
                        Case MW_RESULT_BUSY
                            Msg$ = "The Line is Busy"
                            GoSub ProcessTimeout
                    End Select
                    Call MWStatus(0, Msg$)
                End If
            Case MW_SCRIPT_CMD_TIMEOUT '--------------------------- Set the Timeout value
                If Len(NextLine$) > 7 Then
                    TimeOut = Val(Mid$(NextLine$, 8))
                End If
            Case MW_SCRIPT_CMD_WAITFOR '--------------------------- Wait for a string
                '-- Is the port open?
                If Comm1.PortOpen = False Then
                    MWSetError MW_ER_PORTCLOSED
                    GoTo PSExitHere
                End If
                If Len(NextLine$) > 7 Then
                    Wait$ = Mid$(NextLine$, 8)
                    Wait$ = ScriptProcessData$(Wait$)
                    If Len(Wait$) = 0 Then
                        '-- No string was specified
                        MWSetError MW_SCRIPTPLAY_BAD_COMMAND
                        Exit For
                    End If
                    '-- Status = Waiting
                    Call MWStatus(MW_STAT_SCRIPT_WAITFOR, Wait$)
                    '-- Wait for the string
                    CDFlag = False
                    ExcludeFlag = False
                    Received$ = WaitFor$(Comm1, Wait$, TimeOut, CDFlag, ExcludeFlag)
                    '-- Write to the capture file.
                    If MW_CaptureFile Then
                        Print #MW_CaptureFile, Received$;
                    End If
                    '-- Error?
                    Select Case MWError()
                        Case MW_ER_TIMEOUT
                            GoSub ProcessTimeout
                            Exit For
                        Case Is <> MW_WAITFOR_OK
                            If MW_DisplayFlag Then
                                If Comm1.Emulation Then
                                    OldLen = Comm1.InputLen
                                    Comm1.InputLen = 0
                                    In$ = Comm1.Input
                                    Comm1.Disp = In$
                                    Comm1.InputLen = OldLen
                                End If
                            End If
                            Exit For
                    End Select
                End If
            Case MW_SCRIPT_CMD_SEND '------------------------------ Send a string.
                '-- Is the port open?
                If Comm1.PortOpen = False Then
                    MWSetError MW_ER_PORTCLOSED
                    GoTo PSExitHere
                End If
                If Len(NextLine$) > 4 Then
                    OutString$ = Mid$(NextLine$, 5)
                    OutString$ = ScriptProcessData$(OutString$)
                    '-- Send away
                    Comm1.Output = OutString$
                End If
            Case MW_SCRIPT_CMD_HANGUP '---------------------------- Hang up the phone.
                '-- Is the port open?
                If Comm1.PortOpen = False Then
                    MWSetError MW_ER_PORTCLOSED
                    GoTo PSExitHere
                End If
                '-- Status = Hanging up
                Call MWStatus(MW_STAT_SCRIPT_HANGUP, "")
                '-- Hangup
                Hangup Comm1, 2, True
                If MWError() <> MW_HANGUP_OK Then
                    Exit For
                End If
            Case MW_SCRIPT_CMD_PROTOCOL '-------------------------- Set the transfer protocol.
                If Len(NextLine$) > 8 Then
                    Prtcl$ = Mid$(NextLine$, 9)
                    Prtcl$ = UCase$(Trim$(Prtcl$))
                    Prtcl$ = ScriptProcessData$(Prtcl$)
                    '-- Set the protocol
                    Select Case Prtcl$
                        Case "XMODEM-CHECKSUM"
                            Comm1.XferProtocol = PDQ_XMODEM_CHECKSUM
                        Case "XMODEM-CRC"
                            Comm1.XferProtocol = PDQ_XMODEM_CRC
                        Case "XMODEM-1K"
                            Comm1.XferProtocol = PDQ_XMODEM_1K
                        Case "YMODEM-G"
                            Comm1.XferProtocol = PDQ_YMODEM_G
                        Case "YMODEM-BATCH"
                            Comm1.XferProtocol = PDQ_YMODEM_BATCH
                        Case "ZMODEM"
                            Comm1.XferProtocol = PDQ_ZMODEM
                        Case "KERMIT"
                            Comm1.XferProtocol = PDQ_KERMIT
                        Case "COMPUSERVE"
                            Comm1.XferProtocol = PDQ_COMPUSERVE_BPLUS
                        Case Else
                            MWSetError MW_SCRIPTPLAY_BAD_PROTOCOL
                            Exit For
                    End Select
                    ProtocolSetFlag = True
                End If
            Case MW_SCRIPT_CMD_UPLOAD '---------------------------- Upload (Send) a file.
                '-- Is the port open?
                If Comm1.PortOpen = False Then
                    MWSetError MW_ER_PORTCLOSED
                    GoTo PSExitHere
                End If
                '-- Has the protocol been set?
                If Not ProtocolSetFlag Then
                    MWSetError MW_SCRIPTPLAY_NOPROTOCOL
                    GoTo PSExitHere
                End If
                If Len(NextLine$) > 6 Then
                    XFileName$ = Mid$(NextLine$, 7)
                    XFileName$ = ScriptProcessData$(XFileName$)
                    '-- Test for file validity (if protocol requires a filename)
                    Select Case Comm1.XferProtocol
                        Case Is <> PDQ_COMPUSERVE_BPLUS, PDQ_KERMIT
                            On Error Resume Next
                            FileNum = FreeFile
                            Open XFileName$ For Input As FileNum
                            E = Err
                            Close FileNum
                            If E Then
                                MWSetError MW_SCRIPTPLAY_BAD_XFERFILE
                                Exit For
                            End If
                    End Select
                End If
                '-- Use the dialog box
                Comm1.XferStatusDialog = PDQ_XFERDIALOG_MODELESS
                '-- Status = Uploading
                Call MWStatus(MW_STAT_SCRIPT_UPLOAD, XFileName$)
                '-- Upload
                Comm1.Upload = XFileName$
                '-- Wait until the transfer has started
                Do
                    DoEvents
                    If MW_CancelFlag Then
                        MWSetError MW_ER_CANCELED
                        Exit Do
                    End If
                Loop Until Comm1.XferStatus <> PDQ_XFER_TERM_OK
                '-- Wait until the transfer has finished, poll every 2 seconds
                Do
                    MWPause 2
                    If Comm1.XferStatus = PDQ_XFER_TERM_OK Then Exit Do
                    If Comm1.XferStatus = PDQ_XFER_TERM_ERROR Then Exit Do
                Loop
            Case MW_SCRIPT_CMD_DOWNLOAD '-------------------------- Download (Receive) a file
                '-- Is the port open?
                If Comm1.PortOpen = False Then
                    MWSetError MW_ER_PORTCLOSED
                    GoTo PSExitHere
                End If
                If Len(NextLine$) > 8 Then
                    XFileName$ = Mid$(NextLine$, 9)
                    XFileName$ = ScriptProcessData$(XFileName$)
                End If
                '-- Test for file validity (if protocol requires a filename)
                Select Case Comm1.XferProtocol
                    Case PDQ_XMODEM_CHECKSUM, PDQ_XMODEM_CRC, PDQ_XMODEM_1K
                        On Error Resume Next
                        FileNum = FreeFile
                        Open XFileName$ For Input As FileNum
                        E = Err
                        Close FileNum
                        If E Then
                            MWSetError MW_SCRIPTPLAY_BAD_XFERFILE
                            Exit For
                        End If
                End Select
                '-- Use the dialog box
                Comm1.XferStatusDialog = PDQ_XFERDIALOG_MODELESS
                '-- Download
                Comm1.Download = XFileName$
                '-- Wait until the transfer has started
                Do
                    DoEvents
                    If MW_CancelFlag Then
                        MWSetError MW_ER_CANCELED
                        Exit Do
                    End If
                Loop Until Comm1.XferStatus <> PDQ_XFER_TERM_OK
                '-- Wait until the transfer has finished, poll every 2 seconds
                Do
                    MWPause 2
                    If Comm1.XferStatus = PDQ_XFER_TERM_OK Then Exit Do
                    If Comm1.XferStatus = PDQ_XFER_TERM_ERROR Then Exit Do
                Loop
            Case MW_SCRIPT_CMD_PAUSE '----------------------------- Pause for a number of seconds.
                If Len(NextLine$) > 5 Then
                    NumSecs! = Val(Mid$(NextLine$, 6))
                    If NumSecs! > 0 Then
                        MWPause NumSecs!
                    End If
                End If
            Case MW_SCRIPT_CMD_STOP '------------------------------ Stop VB app for debugging.
                Stop
            Case Else
                MWSetError MW_SCRIPTPLAY_BAD_COMMAND
                Exit For
        End Select

PS_LoopHere:
    
    Next
    
    '-- If all is cool, don't return errors
    If MWError() = MW_SCRIPTPLAY_OK Then
        ErrorLineNum = 0
        ErrorLine$ = ""
    End If

PSExitHere:

    '-- De-allocate the variables array
    Erase MW_ScriptVars, MW_ScriptLabel

    Close FileNum

    '-- Reset AutoProcess and RThreshold
    If AutoP Then
        Comm1.AutoProcess = AutoP
    End If
    If RT Then
        Comm1.RThreshold = RT
    End If

    Exit Sub

ProcessTimeout:

    If Len(TimeoutLabel$) Then
        On Error Resume Next
        T = UBound(MW_ScriptLabel)
        If T Then
            For I = 1 To T
                If MW_ScriptLabel(I).Label = TimeoutLabel$ Then
                    TimeoutLine = MW_ScriptLabel(I).LineNum
                    ErrorLineNum = TimeoutLine
                    Exit For
                End If
            Next
        End If
    End If
    Return

End Sub

Function ScriptProcessData$ (ScriptData$)
'-- Internal function used to process a line of script file text (remove
'   quotes, trim out spaces, etc) and replace variables with their values.

Dim FirstQuote As Integer, SecondQuote As Integer, Index As Integer
Dim Temp$

    '-- Trim
    ScriptData$ = ZTrim$(ScriptData$)

    '-- Find quotes
    FirstQuote = InStr(ScriptData$, Chr$(34))
    If FirstQuote Then
        If FirstQuote < Len(ScriptData$) Then
            Temp$ = Mid$(ScriptData$, FirstQuote + 1)
            SecondQuote = InStr(Temp$, Chr$(34))
            If SecondQuote Then
                ScriptData$ = Mid$(ScriptData$, FirstQuote + 1, SecondQuote - 1)
            Else
                ScriptData$ = Mid$(ScriptData$, FirstQuote + 1)
            End If
        End If
    Else
        '-- Search through the variable array
        Index = ScriptVarIndex(ScriptData$)
        If Len(MW_ScriptVars(Index).Value) Then
            '-- Return the value if it exists
            ScriptData$ = MW_ScriptVars(Index).Value
        Else
            '-- Signify that this is a new variable by returning it's index
            ScriptData$ = "XXX" & LTrim$(Str$(Index))
        End If
    End If
    
    ScriptData$ = RemoveCaret$(ScriptData$)

    ScriptProcessData$ = ScriptData$

End Function

Sub ScriptRaw2File (ScriptFileName$)
'-- Internal: This subroutine writes recorded script data to the file
'   specified by ScriptRecord.

Dim Total As Integer, I As Integer, LastIndex As Integer, FileNum As Integer
Dim Comma As Integer
Dim Pressed$, LastCmd$, Cmd$, Dat$, Out$

    Total = UBound(MW_RecScriptData)
    
    '-- Convert IN and OUT to WAITFOR and SEND
    '   and take out Waitfors that wait for keys pressed
    
    For I = 1 To Total
        If MW_RecScriptData(I).Cmd = "IN" Then MW_RecScriptData(I).Cmd = "WAITFOR"
        If MW_RecScriptData(I).Cmd = "OUT" Then MW_RecScriptData(I).Cmd = "SEND"
        
        If Len(MW_RecScriptData(I).Cmd) Then
        Select Case MW_RecScriptData(I).Cmd
            Case "WAITFOR"
                Do
                    If Len(Pressed$) = 0 Then Exit Do
                    If UCase$(Left$(MW_RecScriptData(I).Dat, 1)) = Left$(Pressed$, 1) Then
                        MW_RecScriptData(I).Dat = Right$(MW_RecScriptData(I).Dat, Len(MW_RecScriptData(I).Dat) - 1)
                    End If
                    Pressed$ = Right$(Pressed$, Len(Pressed$) - 1)
                Loop
                If Len(MW_RecScriptData(I).Dat) = 0 Then
                    MW_RecScriptData(I).Cmd = ""
                End If
            Case "SEND"
                Pressed$ = Pressed$ & UCase$(MW_RecScriptData(I).Dat)
        End Select
        End If
    Next
    
    '-- Concatonate IN and OUT commands that occur sequentially
    For I = 1 To Total
        
        If Len(MW_RecScriptData(I).Cmd) Then
            If MW_RecScriptData(I).Cmd = "WAITFOR" Or MW_RecScriptData(I).Cmd = "SEND" Then
                If MW_RecScriptData(I).Cmd = LastCmd$ Then
                    MW_RecScriptData(LastIndex).Dat = MW_RecScriptData(LastIndex).Dat & MW_RecScriptData(I).Dat
                    MW_RecScriptData(I).Cmd = ""
                Else
                    LastCmd$ = MW_RecScriptData(I).Cmd
                    LastIndex = I
                End If
            Else
                LastCmd$ = MW_RecScriptData(I).Cmd
                LastIndex = I
            End If
        End If
    Next

    '-- write to the file
    FileNum = FreeFile
    Open ScriptFileName$ For Output As FileNum
    
    For I = 1 To Total
        If Len(MW_RecScriptData(I).Cmd) Then
            If MW_RecScriptData(I).Cmd = "WAITFOR" Then
                If MW_ScriptWaitLen Then
                    MW_RecScriptData(I).Dat = Right$(MW_RecScriptData(I).Dat, MW_ScriptWaitLen)
                End If
            End If
            Cmd$ = AddCaret$(MW_RecScriptData(I).Cmd)
            Dat$ = AddCaret$(MW_RecScriptData(I).Dat)
            Out$ = Cmd$ & " "
            Select Case Cmd$
                Case "WAITFOR", "SEND", "PROTOCOL", "SETTINGS", "DOWNLOAD", "UPLOAD", "CAPTURE"
                    Out$ = Out$ & Chr$(34) & Dat$ & Chr$(34)
                Case "DIAL"
                    Comma = InStr(Dat$, ",")
                    If Comma Then
                        Out$ = Out$ & Chr$(34) & Left$(Dat$, Comma - 1) & Chr$(34) & Mid$(Dat$, Comma)
                    Else
                        Out$ = Out$ & Chr$(34) & Dat$ & Chr$(34)
                    End If
                Case Else
                    Out$ = Out$ & Dat$
            End Select

            Print #FileNum, Out$
        End If
    Next
    Print #FileNum, ":TIMEOUT"
    Print #FileNum, "END"
    Close FileNum

End Sub

Sub ScriptRecord (Comm1 As Control, FileName$, Length)
'-- This routine opens a script file to be recorded.
'   All ModemWare routines write their actions to a script
'   array if a script is being recorded. It's built in!

    Static OldAutoProcess, OldRThreshold, ScriptFileName$
    If Len(FileName$) = 0 Then
        If MW_ScriptRecording Then
            Comm1.AutoProcess = OldAutoProcess
            Comm1.RThreshold = OldRThreshold
            MW_ScriptRecording = False
            Call ScriptRaw2File(ScriptFileName$)
            ScriptFileName$ = ""
        Else
            MsgBox "Error - ScriptRecord called with "" when no script was being recorded"
        End If
        Exit Sub
    Else
        ScriptFileName$ = FileName$
        MW_ScriptWaitLen = Length
        OldAutoProcess = Comm1.AutoProcess
        OldRThreshold = Comm1.RThreshold
        ReDim MW_RecScriptData(1 To 3) As MW_ScriptDataType
        MW_RecScriptData(1).Cmd$ = "PORT"
        MW_RecScriptData(1).Dat$ = Trim$(Str$(Comm1.CommPort))
        MW_RecScriptData(2).Cmd$ = "TIMEOUT"
        MW_RecScriptData(2).Dat$ = "60"
        MW_RecScriptData(3).Cmd$ = "ON_TIMEOUT_GOTO"
        MW_RecScriptData(3).Dat$ = ":TIMEOUT"

        Comm1.AutoProcess = PDQ_AUTOPROCESS_NONE
        Comm1.RThreshold = 1
        MW_ScriptRecording = True
    End If

End Sub

Sub ScriptRecordKeyPress (Pressed$)
'-- Processes a pressed key if a script is being recorded.

    '-- Are we recording a script?
    If MW_ScriptRecording Then
        AddToRecScriptData "OUT", Pressed$
    End If
    
End Sub

Sub ScriptRecordOnComm (In$)
'-- Processes an incoming character if a script is being recorded.

    '-- Are we recording a script?
    If MW_ScriptRecording Then
        AddToRecScriptData "IN", In$
    End If

End Sub

Function ScriptVarIndex (Variable$)
'-- Internal: returns the index in the script variable array of a variable
'   or adds the variable if it is not found.

Dim Elements As Integer, I As Integer

    '-- Get the size of the array
    On Error Resume Next
    Elements = UBound(MW_ScriptVars)
    On Error GoTo 0
    
    '-- Do we have this variable already?
    If Elements Then
        For I = 1 To Elements
            If MW_ScriptVars(I).Variable = Variable$ Then
                ScriptVarIndex = I
                Exit Function
            End If
        Next
    End If

    '-- No, we do not. Add it to the array
    Elements = Elements + 1
    ReDim Preserve MW_ScriptVars(1 To Elements) As MW_ScriptVariableType
    MW_ScriptVars(Elements).Variable = Variable$
    ScriptVarIndex = Elements
    
End Function

